@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2006 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2005-2018 Apple Inc. All rights reserved.
  *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_START@
  * 
@@ -26,7 +26,6 @@
  * @APPLE_OSREFERENCE_LICENSE_HEADER_END@
  */
 
-#define MACH__POSIX_C_SOURCE_PRIVATE 1 /* pulls in suitable savearea from mach/ppc/thread_status.h */
 #include <kern/thread.h>
 #include <mach/thread_status.h>
 
@@ -43,11 +42,16 @@ typedef x86_saved_state_t savearea_t;
 #include <sys/dtrace.h>
 #include <sys/dtrace_impl.h>
 #include <libkern/OSAtomic.h>
+#include <i386/x86_hypercall.h>
 #include <kern/thread_call.h>
 #include <kern/task.h>
 #include <kern/sched_prim.h>
 #include <miscfs/devfs/devfs.h>
 #include <mach/vm_param.h>
+#include <machine/pal_routines.h>
+#include <i386/cpuid.h>
+#include <i386/mp.h>
+#include <machine/trap.h>
 
 /*
  * APPLE NOTE:  The regmap is used to decode which 64bit uregs[] register
@@ -126,11 +130,6 @@ dtrace_getipl(void)
 /*
  * MP coordination
  */
-
-extern void mp_broadcast(
-       void (*action_func)(void *),
-       void *arg);
-
 typedef struct xcArg {
 	processorid_t cpu;
 	dtrace_xcall_t f;
@@ -147,6 +146,7 @@ xcRemote( void *foo )
 	}
 }
 
+
 /*
  * dtrace_xcall() is not called from probe context.
  */
@@ -159,19 +159,28 @@ dtrace_xcall(processorid_t cpu, dtrace_xcall_t f, void *arg)
 	xcArg.f = f;
 	xcArg.arg = arg;
 
-	mp_broadcast( xcRemote, (void *)&xcArg);
+	if (cpu == DTRACE_CPUALL) {
+		mp_cpus_call (CPUMASK_ALL, ASYNC, xcRemote, (void*)&xcArg);
+	}
+	else {
+		mp_cpus_call (cpu_to_cpumask((cpu_t)cpu), ASYNC, xcRemote, (void*)&xcArg);
+	}
 }
 
 /*
  * Runtime and ABI
  */
-
 uint64_t
 dtrace_getreg(struct regs *savearea, uint_t reg)
 {
 	boolean_t is64Bit = proc_is64bit(current_proc());
 	x86_saved_state_t *regs = (x86_saved_state_t *)savearea;
 
+	if (regs == NULL) {
+		DTRACE_CPUFLAG_SET(CPU_DTRACE_ILLOP);
+		return (0);
+	}
+
 	if (is64Bit) {
 	    if (reg <= SS) {
 		reg = regmap[reg];
@@ -245,6 +254,184 @@ dtrace_getreg(struct regs *savearea, uint_t reg)
 	}
 }
 
+uint64_t
+dtrace_getvmreg(uint_t ndx)
+{
+	uint64_t reg = 0;
+	bool failed = false;
+
+	/* Any change in the vmread final opcode must be reflected in dtrace_handle_trap below. */
+	__asm__ __volatile__(
+		"vmread %2, %0\n"
+		"ja 1f\n"
+		"mov $1, %1\n"
+		"1:\n"
+	: "=a" (reg), "+r" (failed) : "D" ((uint64_t)ndx));
+
+	/*
+	 * Check for fault in vmreg first. If DTrace has recovered the fault cause by
+	 * vmread above then the value in failed will be unreliable.
+	 */
+	if (DTRACE_CPUFLAG_ISSET(CPU_DTRACE_ILLOP)) {
+		return 0;
+	}
+
+	/* If vmread succeeded but failed because CF or ZS is 1 report fail. */
+	if (failed) {
+		DTRACE_CPUFLAG_SET(CPU_DTRACE_BADADDR);
+		cpu_core[CPU->cpu_id].cpuc_dtrace_illval = ndx;
+		return 0;
+	}
+
+	return reg;
+}
+
+static void
+dtrace_vmcall(x86_saved_state_t *regs, uint64_t *rflags)
+{
+	uint64_t flags = 0;
+
+	/*
+	 * No constraints available for r8 or r9 which means they must be
+	 * handled explicitly.
+	 */
+	__asm__ volatile (
+	     "   movq %12, %%r8  \n"
+	     "   movq %13, %%r9  \n"
+	     "   vmcall          \n"
+	     "   movq %%r8, %5   \n"
+	     "   movq %%r9, %6   \n"
+	     "   pushfq          \n"
+	     "   popq %7         \n"
+
+	    : "=a" (regs->ss_64.rax),
+	      "=D" (regs->ss_64.rdi),
+	      "=S" (regs->ss_64.rsi),
+	      "=d" (regs->ss_64.rdx),
+	      "=c" (regs->ss_64.rcx),
+	      "=r" (regs->ss_64.r8),  /* %5 */
+	      "=r" (regs->ss_64.r9),  /* %6 */
+	      "=r" (flags)            /* %7 */
+
+	    : "a"  (regs->ss_64.rax),
+	      "D"  (regs->ss_64.rdi),
+	      "S"  (regs->ss_64.rsi),
+	      "d"  (regs->ss_64.rdx),
+	      "c"  (regs->ss_64.rcx),
+	      "r"  (regs->ss_64.r8),  /* %12 */
+	      "r"  (regs->ss_64.r9)   /* %13 */
+
+	    : "memory", "r8", "r9");
+
+	*rflags = flags;
+
+	return;
+}
+
+static inline void
+dtrace_cpuid(x86_saved_state_t *regs)
+{
+	__asm__ volatile (
+	     "cpuid"
+	    : "=a" (regs->ss_64.rax),
+	      "=b" (regs->ss_64.rbx),
+	      "=c" (regs->ss_64.rcx),
+	      "=d" (regs->ss_64.rdx)
+
+	    : "a"  (regs->ss_64.rax),
+	      "b"  (regs->ss_64.rbx),
+	      "c"  (regs->ss_64.rcx),
+	      "d"  (regs->ss_64.rdx));
+}
+
+static bool
+dtrace_applepv_available(uint64_t flag)
+{
+	static bool checked = false;
+	static uint64_t features = 0;
+
+	if (checked) {
+		return (features & flag) != 0;
+	}
+
+	x86_saved_state_t regs = {0};
+
+	regs.ss_64.rax = 1;
+	dtrace_cpuid(&regs);
+
+	/* Bit 31 - HV bit. */
+	if ((regs.ss_64.rcx & _Bit(31)) != 0) {
+		for (uint32_t base = 0x40000100; base < 0x40010000; base += 0x100) {
+			regs.ss_64.rax = base;
+			dtrace_cpuid(&regs);
+
+			/* "apple-pv-xnu" */
+			if (regs.ss_64.rbx != 0x6c707061 ||
+			    regs.ss_64.rcx != 0x76702d65 ||
+			    regs.ss_64.rdx != 0x756e782d) {
+				continue;
+			}
+
+			uint64_t feature_leaf = regs.ss_64.rax;
+
+			regs.ss_64.rax = base + APPLEPV_INTERFACE_LEAF_INDEX;
+			dtrace_cpuid(&regs);
+
+			/* "AH#1" */
+			if (regs.ss_64.rax != 0x31234841) {
+				continue;
+			}
+
+			/* Find features. */
+			regs.ss_64.rax = feature_leaf;
+			dtrace_cpuid(&regs);
+
+			features = regs.ss_64.rdx;
+			break;
+		}
+	}
+
+	checked = true;
+	return (features & flag) != 0;
+}
+
+void
+dtrace_livedump(char *filename, size_t len)
+{
+	x86_saved_state_t regs = {
+	    .ss_64.rax = HVG_HCALL_CODE(HVG_HCALL_TRIGGER_DUMP),
+	    .ss_64.rdi = HVG_HCALL_DUMP_OPTION_REGULAR,
+	};
+
+	if (len > 0) {
+		filename[0] = '\0';
+	}
+
+	if (!dtrace_applepv_available(CPUID_LEAF_FEATURE_COREDUMP)) {
+		DTRACE_CPUFLAG_SET(CPU_DTRACE_ILLOP);
+		return;
+	}
+
+	uint64_t rflags = 0;
+	dtrace_vmcall(&regs, &rflags);
+	if ((rflags & EFL_CF) != 0) {
+		/* An empty filename indicates failure to dump. */
+		return;
+	}
+
+	/* Extract the filename. */
+	char str[57] = {'\0'};
+	memcpy(&str[0],  &regs.ss_64.rax, 8);
+	memcpy(&str[8],  &regs.ss_64.rdi, 8);
+	memcpy(&str[16], &regs.ss_64.rsi, 8);
+	memcpy(&str[24], &regs.ss_64.rdx, 8);
+	memcpy(&str[32], &regs.ss_64.rcx, 8);
+	memcpy(&str[40], &regs.ss_64.r8,  8);
+	memcpy(&str[48], &regs.ss_64.r9,  8);
+
+	(void) strlcpy(filename, str, len);
+}
+
 #define RETURN_OFFSET 4
 #define RETURN_OFFSET64 8
 
@@ -252,10 +439,10 @@ static int
 dtrace_getustack_common(uint64_t *pcstack, int pcstack_limit, user_addr_t pc,
     user_addr_t sp)
 {
-#if 0
 	volatile uint16_t *flags =
 	    (volatile uint16_t *)&cpu_core[CPU->cpu_id].cpuc_dtrace_flags;
 
+#if 0
 	uintptr_t oldcontext = lwp->lwp_oldcontext; /* XXX signal stack crawl */
 	size_t s1, s2;
 #endif
@@ -318,17 +505,11 @@ dtrace_getustack_common(uint64_t *pcstack, int pcstack_limit, user_addr_t pc,
 			}
 		}
 
-#if 0 /* XXX */
-		/*
-		 * This is totally bogus:  if we faulted, we're going to clear
-		 * the fault and break.  This is to deal with the apparently
-		 * broken Java stacks on x86.
-		 */
+		/* Truncate ustack if the iterator causes fault. */
 		if (*flags & CPU_DTRACE_FAULT) {
 			*flags &= ~CPU_DTRACE_FAULT;
 			break;
 		}
-#endif
 	}
 
 	return (ret);
@@ -342,6 +523,7 @@ static int
 dtrace_adjust_stack(uint64_t **pcstack, int *pcstack_limit, user_addr_t *pc,
                     user_addr_t sp)
 {
+    volatile uint16_t *flags = (volatile uint16_t *) &cpu_core[CPU->cpu_id].cpuc_dtrace_flags;
     int64_t missing_tos;
     int rc = 0;
     boolean_t is64Bit = proc_is64bit(current_proc());
@@ -366,6 +548,11 @@ dtrace_adjust_stack(uint64_t **pcstack, int *pcstack_limit, user_addr_t *pc,
             *pc = dtrace_fuword64(sp);
         else
             *pc = dtrace_fuword32(sp);
+
+	/* Truncate ustack if the iterator causes fault. */
+	if (*flags & CPU_DTRACE_FAULT) {
+		*flags &= ~CPU_DTRACE_FAULT;
+	}
     } else {
         /*
          * We might have a top of stack override, in which case we just
@@ -420,11 +607,12 @@ dtrace_getupcstack(uint64_t *pcstack, int pcstack_limit)
 	if (thread == NULL)
 		goto zero;
 
+	pal_register_cache_state(thread, VALID);
 	regs = (x86_saved_state_t *)find_user_regs(thread);
 	if (regs == NULL)
 		goto zero;
 		
-	*pcstack++ = (uint64_t)proc_selfpid();
+	*pcstack++ = (uint64_t)dtrace_proc_selfpid();
 	pcstack_limit--;
 
 	if (pcstack_limit <= 0)
@@ -483,6 +671,7 @@ dtrace_getustackdepth(void)
 	if (DTRACE_CPUFLAG_ISSET(CPU_DTRACE_FAULT))
 		return (-1);
 
+	pal_register_cache_state(thread, VALID);
 	regs = (x86_saved_state_t *)find_user_regs(thread);
 	if (regs == NULL)
 		return 0;
@@ -550,7 +739,7 @@ dtrace_getufpstack(uint64_t *pcstack, uint64_t *fpstack, int pcstack_limit)
 	if (regs == NULL)
 		goto zero;
 		
-	*pcstack++ = (uint64_t)proc_selfpid();
+	*pcstack++ = (uint64_t)dtrace_proc_selfpid();
 	pcstack_limit--;
 
 	if (pcstack_limit <= 0)
@@ -622,17 +811,11 @@ dtrace_getufpstack(uint64_t *pcstack, uint64_t *fpstack, int pcstack_limit)
 			}
 		}
 
-#if 0 /* XXX */
-		/*
-		 * This is totally bogus:  if we faulted, we're going to clear
-		 * the fault and break.  This is to deal with the apparently
-		 * broken Java stacks on x86.
-		 */
+		/* Truncate ustack if the iterator causes fault. */
 		if (*flags & CPU_DTRACE_FAULT) {
 			*flags &= ~CPU_DTRACE_FAULT;
 			break;
 		}
-#endif
 	}
 
 zero:
@@ -666,11 +849,7 @@ dtrace_getpcstack(pc_t *pcstack, int pcstack_limit, int aframes,
 
 	while (depth < pcstack_limit) {
 		nextfp = *(struct frame **)fp;
-#if defined(__x86_64__)
 		pc = *(uintptr_t *)(((uintptr_t)fp) + RETURN_OFFSET64);
-#else
-		pc = *(uintptr_t *)(((uintptr_t)fp) + RETURN_OFFSET);
-#endif
 
 		if (nextfp <= minfp || nextfp >= stacktop) {
 			if (on_intr) {
@@ -725,40 +904,28 @@ struct frame {
 };
 
 uint64_t
-dtrace_getarg(int arg, int aframes)
+dtrace_getarg(int arg, int aframes, dtrace_mstate_t *mstate, dtrace_vstate_t *vstate)
 {
-	uint64_t val;
+	uint64_t val = 0;
 	struct frame *fp = (struct frame *)__builtin_frame_address(0);
 	uintptr_t *stack;
 	uintptr_t pc;
 	int i;
 
 
-#if defined(__x86_64__)
     /*
      * A total of 6 arguments are passed via registers; any argument with
      * index of 5 or lower is therefore in a register.
      */
     int inreg = 5;
-#endif
 
 	for (i = 1; i <= aframes; i++) {
 		fp = fp->backchain;
 		pc = fp->retaddr;
 
-		if (pc  == (uintptr_t)dtrace_invop_callsite) {
-#if defined(__i386__)
-			/*
-			 * If we pass through the invalid op handler, we will
-			 * use the pointer that it passed to the stack as the
-			 * second argument to dtrace_invop() as the pointer to
-			 * the frame we're hunting for.
-			 */
-
-			stack = (uintptr_t *)&fp[1]; /* Find marshalled arguments */
-			fp = (struct frame *)stack[1]; /* Grab *second* argument */
-			stack = (uintptr_t *)&fp[1]; /* Find marshalled arguments */
-#elif defined(__x86_64__)
+		if (dtrace_invop_callsite_pre != NULL
+			&& pc  >  (uintptr_t)dtrace_invop_callsite_pre
+			&& pc  <= (uintptr_t)dtrace_invop_callsite_post) {
 			/*
 			 * In the case of x86_64, we will use the pointer to the
 			 * save area structure that was pushed when we took the
@@ -781,24 +948,26 @@ dtrace_getarg(int arg, int aframes)
 			x86_saved_state64_t *saved_state = saved_state64(tagged_regs);
 
 			if (arg <= inreg) {
-				stack = (uintptr_t *)&saved_state->rdi;
+				stack = (uintptr_t *)(void*)&saved_state->rdi;
 			} else {
-				stack = (uintptr_t *)(saved_state->isf.rsp);
-				arg -= inreg;
+				fp = (struct frame *)(saved_state->isf.rsp);
+				stack = (uintptr_t *)&fp[1]; /* Find marshalled
+								arguments */
+				arg -= inreg + 1;
 			}
-#else
-#error Unknown arch
-#endif
 			goto load;
 		}
 	}
 
 	/*
-	 * Arrive here when provider has called dtrace_probe directly.
+	 * We know that we did not come through a trap to get into
+	 * dtrace_probe() --  We arrive here when the provider has
+	 * called dtrace_probe() directly.
+	 * The probe ID is the first argument to dtrace_probe().
+	 * We must advance beyond that to get the argX.
 	 */
 	arg++; /* Advance past probeID */
 
-#if defined(__x86_64__)
 	if (arg <= inreg) {
 		/*
 		 * This shouldn't happen.  If the argument is passed in a
@@ -810,13 +979,14 @@ dtrace_getarg(int arg, int aframes)
 	}
 
 	arg -= (inreg + 1);
-#endif
 	stack = (uintptr_t *)&fp[1]; /* Find marshalled arguments */
 
 load:
-	DTRACE_CPUFLAG_SET(CPU_DTRACE_NOFAULT);
-	val = *(((uint64_t *)stack) + arg); /* dtrace_probe arguments arg0 .. arg4 are 64bits wide */
-	DTRACE_CPUFLAG_CLEAR(CPU_DTRACE_NOFAULT);
+	if (dtrace_canload((uint64_t)(stack + arg), sizeof(uint64_t),
+		mstate, vstate)) {
+		/* dtrace_probe arguments arg0 ... arg4 are 64bits wide */
+		val = dtrace_load64((uint64_t)(stack + arg));
+	}
 
 	return (val);
 }
@@ -836,3 +1006,34 @@ dtrace_toxic_ranges(void (*func)(uintptr_t base, uintptr_t limit))
 			func(VM_MAX_KERNEL_ADDRESS + 1, ~(uintptr_t)0);
 }
 
+/*
+ * Trap Safety
+ */
+extern boolean_t dtrace_handle_trap(int, x86_saved_state_t *);
+
+boolean_t
+dtrace_handle_trap(int trapno, x86_saved_state_t *state)
+{
+	x86_saved_state64_t *saved_state = saved_state64(state);
+
+	if (!DTRACE_CPUFLAG_ISSET(CPU_DTRACE_NOFAULT)) {
+		return FALSE;
+	}
+
+	/*
+	 * General purpose solution would require pulling in disassembler. Right now there
+	 * is only one specific case to be handled so it is hardcoded here.
+	 */
+	if (trapno == T_INVALID_OPCODE) {
+		uint8_t *inst = (uint8_t *)saved_state->isf.rip;
+
+		/* vmread %rdi, %rax */
+		if (inst[0] == 0x0f && inst[1] == 0x78 && inst[2] == 0xf8) {
+			DTRACE_CPUFLAG_SET(CPU_DTRACE_ILLOP);
+			saved_state->isf.rip += 3;
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
