@@ -2,7 +2,7 @@
  * Copyright (c) 2000-2007 Apple Inc. All rights reserved.
  *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_START@
- * 
+ *
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -11,10 +11,10 @@
  * unlawful or unlicensed copies of an Apple operating system, or to
  * circumvent, violate, or enable the circumvention or violation of, any
  * terms of an Apple operating system software license agreement.
- * 
+ *
  * Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this file.
- * 
+ *
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -22,7 +22,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- * 
+ *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_END@
  */
 /*
@@ -30,7 +30,7 @@
  *	All Rights Reserved.
  */
 /*
- * posix_shm.c : Support for POSIX semaphore APIs
+ * posix_sem.c : Support for POSIX semaphore APIs
  *
  *	File:	posix_sem.c
  *	Author:	Ananthakrishna Ramesh
@@ -83,59 +83,59 @@
 #include <kern/clock.h>
 #include <mach/kern_return.h>
 
+#define f_flag fp_glob->fg_flag
+#define f_ops fp_glob->fg_ops
 
-#define f_flag f_fglob->fg_flag
-#define f_type f_fglob->fg_type
-#define f_msgcount f_fglob->fg_msgcount
-#define f_cred f_fglob->fg_cred
-#define f_ops f_fglob->fg_ops
-#define f_offset f_fglob->fg_offset
-#define f_data f_fglob->fg_data
-#define	PSEMNAMLEN	31	/* maximum name segment length we bother with */
+#define PSEMNAMLEN      31      /* maximum name segment length we bother with */
 
 struct pseminfo {
-	unsigned int	psem_flags;
-	unsigned int	psem_usecount;
-	mode_t		psem_mode;
-	uid_t		psem_uid;
-	gid_t		psem_gid;
-	char		psem_name[PSEMNAMLEN + 1];	/* segment name */
-	semaphore_t	psem_semobject;
-	proc_t		sem_proc;
-	struct label *	psem_label;
+	unsigned int    psem_flags;
+	unsigned int    psem_usecount;
+	mode_t          psem_mode;
+	uid_t           psem_uid;
+	gid_t           psem_gid;
+	char            psem_name[PSEMNAMLEN + 1];      /* segment name */
+	semaphore_t     psem_semobject;
+	struct label *  psem_label;
+	pid_t           psem_creator_pid;
+	uint64_t        psem_creator_uniqueid;
 };
 #define PSEMINFO_NULL (struct pseminfo *)0
 
-#define	PSEM_NONE	1
-#define	PSEM_DEFINED	2
-#define	PSEM_ALLOCATED	4
-#define	PSEM_MAPPED	8
-#define	PSEM_INUSE	0x10
-#define	PSEM_REMOVED	0x20
-#define	PSEM_INCREATE	0x40
-#define	PSEM_INDELETE	0x80
+#define PSEM_NONE       1
+#define PSEM_DEFINED    2
+#define PSEM_ALLOCATED  4
+#define PSEM_MAPPED     8
+#define PSEM_INUSE      0x10
+#define PSEM_REMOVED    0x20
+#define PSEM_INCREATE   0x40
+#define PSEM_INDELETE   0x80
 
-struct	psemcache {
-	LIST_ENTRY(psemcache) psem_hash;	/* hash chain */
-	struct	pseminfo *pseminfo;		/* vnode the name refers to */
-	int	psem_nlen;		/* length of name */
-	char	psem_name[PSEMNAMLEN + 1];	/* segment name */
+struct  psemcache {
+	LIST_ENTRY(psemcache) psem_hash;        /* hash chain */
+	struct  pseminfo *pseminfo;             /* vnode the name refers to */
+	size_t  psem_nlen;              /* length of name */
+	char    psem_name[PSEMNAMLEN + 1];      /* segment name */
 };
 #define PSEMCACHE_NULL (struct psemcache *)0
 
-struct	psemstats {
-	long	goodhits;		/* hits that we can really use */
-	long	neghits;		/* negative hits that we can use */
-	long	badhits;		/* hits we must drop */
-	long	falsehits;		/* hits with id mismatch */
-	long	miss;		/* misses */
-	long	longnames;		/* long names that ignore cache */
+#define PSEMCACHE_NOTFOUND (0)
+#define PSEMCACHE_FOUND    (-1)
+#define PSEMCACHE_NEGATIVE (ENOENT)
+
+struct  psemstats {
+	long    goodhits;               /* hits that we can really use */
+	long    neghits;                /* negative hits that we can use */
+	long    badhits;                /* hits we must drop */
+	long    falsehits;              /* hits with id mismatch */
+	long    miss;           /* misses */
+	long    longnames;              /* long names that ignore cache */
 };
 
 struct psemname {
-	char	*psem_nameptr;	/* pointer to looked up name */
-	long	psem_namelen;	/* length of looked up component */
-	u_int32_t	psem_hash;	/* hash value of looked up name */
+	char    *psem_nameptr;  /* pointer to looked up name */
+	size_t  psem_namelen;   /* length of looked up component */
+	u_int32_t       psem_hash;      /* hash value of looked up name */
 };
 
 struct psemnode {
@@ -150,64 +150,52 @@ struct psemnode {
 
 #define PSEMHASH(pnp) \
 	(&psemhashtbl[(pnp)->psem_hash & psemhash])
-LIST_HEAD(psemhashhead, psemcache) *psemhashtbl;	/* Hash Table */
-u_long	psemhash;				/* size of hash table - 1 */
-long	psemnument;			/* number of cache entries allocated */
-long	posix_sem_max = 10000;		/* tunable for max POSIX semaphores */
-					/* 10000 limits to ~1M of memory */
-SYSCTL_NODE(_kern, KERN_POSIX, posix, CTLFLAG_RW|CTLFLAG_LOCKED, 0, "Posix");
-SYSCTL_NODE(_kern_posix, OID_AUTO, sem, CTLFLAG_RW|CTLFLAG_LOCKED, 0, "Semaphores");
-SYSCTL_LONG (_kern_posix_sem, OID_AUTO, max, CTLFLAG_RW, &posix_sem_max, "max");
+LIST_HEAD(psemhashhead, psemcache) * psemhashtbl;        /* Hash Table */
+u_long  psemhash;                               /* size of hash table - 1 */
+long    psemnument;                     /* number of cache entries allocated */
+long    posix_sem_max = 10000;          /* tunable for max POSIX semaphores */
+                                        /* 10000 limits to ~1M of memory */
+SYSCTL_NODE(_kern, KERN_POSIX, posix, CTLFLAG_RW | CTLFLAG_LOCKED, 0, "Posix");
+SYSCTL_NODE(_kern_posix, OID_AUTO, sem, CTLFLAG_RW | CTLFLAG_LOCKED, 0, "Semaphores");
+SYSCTL_LONG(_kern_posix_sem, OID_AUTO, max, CTLFLAG_RW | CTLFLAG_LOCKED, &posix_sem_max, "max");
 
-struct psemstats psemstats;		/* cache effectiveness statistics */
+struct psemstats psemstats;             /* cache effectiveness statistics */
 
-static int psem_access(struct pseminfo *pinfo, int mode, kauth_cred_t cred);
+static int psem_access(struct pseminfo *pinfo, mode_t mode, kauth_cred_t cred);
 static int psem_cache_search(struct pseminfo **,
-				struct psemname *, struct psemcache **);
+    struct psemname *, struct psemcache **);
 static int psem_delete(struct pseminfo * pinfo);
 
-static int psem_read (struct fileproc *fp, struct uio *uio,
-			    int flags, vfs_context_t ctx);
-static int psem_write (struct fileproc *fp, struct uio *uio,
-			    int flags, vfs_context_t ctx);
-static int psem_ioctl (struct fileproc *fp, u_long com,
-			    caddr_t data, vfs_context_t ctx);
-static int psem_select (struct fileproc *fp, int which, void *wql, vfs_context_t ctx);
-static int psem_closefile (struct fileglob *fp, vfs_context_t ctx);
+static int psem_closefile(struct fileglob *fp, vfs_context_t ctx);
+static int psem_unlink_internal(struct pseminfo *pinfo, struct psemcache *pcache);
 
-static int psem_kqfilter (struct fileproc *fp, struct knote *kn, vfs_context_t ctx);
+static const struct fileops psemops = {
+	.fo_type     = DTYPE_PSXSEM,
+	.fo_read     = fo_no_read,
+	.fo_write    = fo_no_write,
+	.fo_ioctl    = fo_no_ioctl,
+	.fo_select   = fo_no_select,
+	.fo_close    = psem_closefile,
+	.fo_drain    = fo_no_drain,
+	.fo_kqfilter = fo_no_kqfilter,
+};
 
-struct 	fileops psemops =
-	{ psem_read, psem_write, psem_ioctl, psem_select, psem_closefile, psem_kqfilter, NULL };
+static LCK_GRP_DECLARE(psx_sem_subsys_lck_grp, "posix semaphores");
+static LCK_MTX_DECLARE(psx_sem_subsys_mutex, &psx_sem_subsys_lck_grp);
 
-
-static lck_grp_t       *psx_sem_subsys_lck_grp;
-static lck_grp_attr_t  *psx_sem_subsys_lck_grp_attr;
-static lck_attr_t      *psx_sem_subsys_lck_attr;
-static lck_mtx_t        psx_sem_subsys_mutex;
-
-#define PSEM_SUBSYS_LOCK() lck_mtx_lock(& psx_sem_subsys_mutex)
-#define PSEM_SUBSYS_UNLOCK() lck_mtx_unlock(& psx_sem_subsys_mutex)
+#define PSEM_SUBSYS_LOCK() lck_mtx_lock(&psx_sem_subsys_mutex)
+#define PSEM_SUBSYS_UNLOCK() lck_mtx_unlock(&psx_sem_subsys_mutex)
+#define PSEM_SUBSYS_ASSERT_HELD() LCK_MTX_ASSERT(&psx_sem_subsys_mutex, LCK_MTX_ASSERT_OWNED)
 
 
 static int psem_cache_add(struct pseminfo *psemp, struct psemname *pnp, struct psemcache *pcp);
-/* Initialize the mutex governing access to the posix sem subsystem */
-__private_extern__ void
-psem_lock_init( void )
-{
-
-    psx_sem_subsys_lck_grp_attr = lck_grp_attr_alloc_init();
-
-    psx_sem_subsys_lck_grp = lck_grp_alloc_init("posix shared memory", psx_sem_subsys_lck_grp_attr);
-
-    psx_sem_subsys_lck_attr = lck_attr_alloc_init();
-    lck_mtx_init(& psx_sem_subsys_mutex, psx_sem_subsys_lck_grp, psx_sem_subsys_lck_attr);
-}
+static void psem_cache_delete(struct psemcache *pcp);
+int psem_cache_purge_all(void);
 
 /*
- * Lookup an entry in the cache 
- * 
- * 
+ * Lookup an entry in the cache
+ *
+ *
  * status of -1 is returned if matches
  * If the lookup determines that the name does not exist
  * (negative cacheing), a status of ENOENT is returned. If the lookup
@@ -216,36 +204,37 @@ psem_lock_init( void )
 
 static int
 psem_cache_search(struct pseminfo **psemp, struct psemname *pnp,
-		  struct psemcache **pcache)
+    struct psemcache **pcache)
 {
 	struct psemcache *pcp, *nnp;
 	struct psemhashhead *pcpp;
 
 	if (pnp->psem_namelen > PSEMNAMLEN) {
 		psemstats.longnames++;
-		return (0);
+		return PSEMCACHE_NOTFOUND;
 	}
 
 	pcpp = PSEMHASH(pnp);
 	for (pcp = pcpp->lh_first; pcp != 0; pcp = nnp) {
 		nnp = pcp->psem_hash.le_next;
 		if (pcp->psem_nlen == pnp->psem_namelen &&
-		    !bcmp(pcp->psem_name, pnp->psem_nameptr, 						(u_int)pcp-> psem_nlen))
+		    !bcmp(pcp->psem_name, pnp->psem_nameptr, pcp->psem_nlen)) {
 			break;
+		}
 	}
 
 	if (pcp == 0) {
 		psemstats.miss++;
-		return (0);
+		return PSEMCACHE_NOTFOUND;
 	}
 
 	/* We found a "positive" match, return the vnode */
-        if (pcp->pseminfo) {
+	if (pcp->pseminfo) {
 		psemstats.goodhits++;
 		/* TOUCH(ncp); */
 		*psemp = pcp->pseminfo;
 		*pcache = pcp;
-		return (-1);
+		return PSEMCACHE_FOUND;
 	}
 
 	/*
@@ -253,7 +242,7 @@ psem_cache_search(struct pseminfo **psemp, struct psemname *pnp,
 	 * The nc_vpid field records whether this is a whiteout.
 	 */
 	psemstats.neghits++;
-	return (ENOENT);
+	return PSEMCACHE_NEGATIVE;
 }
 
 /*
@@ -267,17 +256,19 @@ psem_cache_add(struct pseminfo *psemp, struct psemname *pnp, struct psemcache *p
 	struct psemcache *dpcp;
 
 #if DIAGNOSTIC
-	if (pnp->psem_namelen > PSEMNAMLEN)
+	if (pnp->psem_namelen > PSEMNAMLEN) {
 		panic("cache_enter: name too long");
+	}
 #endif
 
 
 	/*  if the entry has already been added by some one else return */
-	if (psem_cache_search(&dpinfo, pnp, &dpcp) == -1) {
-		return(EEXIST);
+	if (psem_cache_search(&dpinfo, pnp, &dpcp) == PSEMCACHE_FOUND) {
+		return EEXIST;
+	}
+	if (psemnument >= posix_sem_max) {
+		return ENOSPC;
 	}
-	if (psemnument >= posix_sem_max)
-		return(ENOSPC);
 	psemnument++;
 	/*
 	 * Fill in cache info, if vp is NULL this is a "negative" cache entry.
@@ -287,19 +278,21 @@ psem_cache_add(struct pseminfo *psemp, struct psemname *pnp, struct psemcache *p
 	 */
 	pcp->pseminfo = psemp;
 	pcp->psem_nlen = pnp->psem_namelen;
-	bcopy(pnp->psem_nameptr, pcp->psem_name, (unsigned)pcp->psem_nlen);
+	bcopy(pnp->psem_nameptr, pcp->psem_name, pcp->psem_nlen);
 	pcpp = PSEMHASH(pnp);
 #if DIAGNOSTIC
 	{
 		struct psemcache *p;
 
-		for (p = pcpp->lh_first; p != 0; p = p->psem_hash.le_next)
-			if (p == pcp)
+		for (p = pcpp->lh_first; p != 0; p = p->psem_hash.le_next) {
+			if (p == pcp) {
 				panic("psem:cache_enter duplicate");
+			}
+		}
 	}
 #endif
 	LIST_INSERT_HEAD(pcpp, pcp, psem_hash);
-	return(0);
+	return 0;
 }
 
 /*
@@ -308,45 +301,76 @@ psem_cache_add(struct pseminfo *psemp, struct psemname *pnp, struct psemcache *p
 void
 psem_cache_init(void)
 {
-	psemhashtbl = hashinit(posix_sem_max / 2, M_SHM, &psemhash);
+	psemhashtbl = hashinit((int)(posix_sem_max / 2), M_SHM, &psemhash);
 }
 
 static void
 psem_cache_delete(struct psemcache *pcp)
 {
 #if DIAGNOSTIC
-	if (pcp->psem_hash.le_prev == 0)
+	if (pcp->psem_hash.le_prev == 0) {
 		panic("psem namecache purge le_prev");
-	if (pcp->psem_hash.le_next == pcp)
+	}
+	if (pcp->psem_hash.le_next == pcp) {
 		panic("namecache purge le_next");
+	}
 #endif /* DIAGNOSTIC */
 	LIST_REMOVE(pcp, psem_hash);
-	pcp->psem_hash.le_prev = NULL;	
+	pcp->psem_hash.le_prev = NULL;
 	psemnument--;
 }
 
-#if NOT_USED
 /*
- * Invalidate a all entries to particular vnode.
- * 
- * We actually just increment the v_id, that will do it. The entries will
- * be purged by lookup as they get found. If the v_id wraps around, we
- * need to ditch the entire cache, to avoid confusion. No valid vnode will
- * ever have (v_id == 0).
+ * Remove all cached psem entries. Open semaphores (with a positive refcount)
+ * will continue to exist, but their cache entries tying them to a particular
+ * name/path will be removed making all future lookups on the name fail.
  */
-static void
-psem_cache_purge(void)
+int
+psem_cache_purge_all(void)
 {
-	struct psemcache *pcp;
+	struct psemcache *pcp, *tmppcp;
 	struct psemhashhead *pcpp;
+	int error = 0;
 
-	for (pcpp = &psemhashtbl[psemhash]; pcpp >= psemhashtbl; pcpp--) {
-		while ( (pcp = pcpp->lh_first) )
-			psem_cache_delete(pcp);
+	if (kauth_cred_issuser(kauth_cred_get()) == 0) {
+		return EPERM;
 	}
-}
-#endif	/* NOT_USED */
 
+	PSEM_SUBSYS_LOCK();
+	for (pcpp = &psemhashtbl[psemhash]; pcpp >= psemhashtbl; pcpp--) {
+		LIST_FOREACH_SAFE(pcp, pcpp, psem_hash, tmppcp) {
+			assert(pcp->psem_nlen);
+			/*
+			 * unconditionally unlink the cache entry
+			 */
+			error = psem_unlink_internal(pcp->pseminfo, pcp);
+			if (error) {
+				goto out;
+			}
+		}
+	}
+	assert(psemnument == 0);
+
+out:
+	PSEM_SUBSYS_UNLOCK();
+
+	if (error) {
+		printf("%s: Error %d removing all semaphores: %ld remain!\n",
+		    __func__, error, psemnument);
+	}
+	return error;
+}
+
+/*
+ *		In order to support unnamed POSIX semaphores, the named
+ *		POSIX semaphores will have to move out of the per-process
+ *		open filetable, and into a global table that is shared with
+ *		unnamed POSIX semaphores, since unnamed POSIX semaphores
+ *		are typically used by declaring instances in shared memory,
+ *		and there's no other way to do this without changing the
+ *		underlying type, which would introduce binary compatibility
+ *		issues.
+ */
 int
 sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 {
@@ -362,15 +386,15 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 	char * nameptr;
 	char * cp;
 	size_t pathlen, plen;
-	int fmode ;
-	int cmode = uap->mode;
+	mode_t fmode;
+	mode_t cmode = (mode_t)uap->mode;
 	int value = uap->value;
 	int incache = 0;
 	struct psemcache *pcp = PSEMCACHE_NULL;
-	kern_return_t kret = KERN_INVALID_ADDRESS;	/* default fail */
-	
+	kern_return_t kret = KERN_INVALID_ADDRESS;      /* default fail */
+
 	AUDIT_ARG(fflags, uap->oflag);
-	AUDIT_ARG(mode, uap->mode);
+	AUDIT_ARG(mode, (mode_t)uap->mode);
 	AUDIT_ARG(value32, uap->value);
 
 	pinfo = PSEMINFO_NULL;
@@ -379,11 +403,7 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 	 * Preallocate everything we might need up front to avoid taking
 	 * and dropping the lock, opening us up to race conditions.
 	 */
-	MALLOC_ZONE(pnbuf, caddr_t, MAXPATHLEN, M_NAMEI, M_WAITOK);
-	if (pnbuf == NULL) {
-		error = ENOSPC;
-		goto bad;
-	}
+	pnbuf = zalloc_flags(ZV_NAMEI, Z_WAITOK | Z_ZERO);
 
 	pathlen = MAXPATHLEN;
 	error = copyinstr(uap->name, pnbuf, MAXPATHLEN, &pathlen);
@@ -391,7 +411,7 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 		goto bad;
 	}
 	AUDIT_ARG(text, pnbuf);
-	if ( (pathlen > PSEMNAMLEN) ) {
+	if ((pathlen > PSEMNAMLEN)) {
 		error = ENAMETOOLONG;
 		goto bad;
 	}
@@ -404,7 +424,7 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 			error = EINVAL;
 			goto bad;
 		}
-        } else {
+	} else {
 		error = EINVAL;
 		goto bad;
 	}
@@ -416,8 +436,8 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 	nd.psem_namelen = plen;
 	nd.psem_hash = 0;
 
-        for (cp = nameptr, i=1; *cp != 0 && i <= plen; i++, cp++) {
-               nd.psem_hash += (unsigned char)*cp * i;
+	for (cp = nameptr, i = 1; *cp != 0 && i <= plen; i++, cp++) {
+		nd.psem_hash += (unsigned char)*cp * i;
 	}
 
 	/*
@@ -425,25 +445,17 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 	 * left unmodified (NULL).
 	 */
 	error = falloc(p, &fp, &indx, vfs_context_current());
-	if (error)
+	if (error) {
 		goto bad;
+	}
 
 	/*
 	 * We allocate a new entry if we are less than the maximum
 	 * allowed and the one at the front of the LRU list is in use.
 	 * Otherwise we use the one at the front of the LRU list.
 	 */
-	MALLOC(pcp, struct psemcache *, sizeof(struct psemcache), M_SHM, M_WAITOK|M_ZERO);
-	if (pcp == PSEMCACHE_NULL) {
-		error = ENOMEM;
-		goto bad;
-	}
-
-	MALLOC(new_pinfo, struct pseminfo *, sizeof(struct pseminfo), M_SHM, M_WAITOK|M_ZERO);
-	if (new_pinfo == NULL) {
-		error = ENOSPC;
-		goto bad;
-	}
+	pcp = kalloc_type(struct psemcache, Z_WAITOK | Z_ZERO | Z_NOFAIL);
+	new_pinfo = kalloc_type(struct pseminfo, Z_WAITOK | Z_ZERO | Z_NOFAIL);
 #if CONFIG_MACF
 	mac_posixsem_label_init(new_pinfo);
 #endif
@@ -454,84 +466,81 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 	 * signal success or failure, which is why we set its default value
 	 * to KERN_INVALID_ADDRESS, above.
 	 */
-	
-	fmode = FFLAGS(uap->oflag);
-	
-	if((fmode & O_CREAT)) {
-		
-		if((value < 0) || (value > SEM_VALUE_MAX)) {
+
+	fmode = (mode_t)FFLAGS(uap->oflag);
+
+	if ((fmode & O_CREAT)) {
+		if ((value < 0) || (value > SEM_VALUE_MAX)) {
 			error = EINVAL;
 			goto bad;
 		}
-		
+
 		kret = semaphore_create(kernel_task, &new_pinfo->psem_semobject, SYNC_POLICY_FIFO, value);
-	
+
 		if (kret != KERN_SUCCESS) {
 			switch (kret) {
-				case KERN_RESOURCE_SHORTAGE:
-					error = ENOMEM;
-					break;
-				case KERN_PROTECTION_FAILURE:
-					error = EACCES;
-					break;
-				default:
-					error = EINVAL;
+			case KERN_RESOURCE_SHORTAGE:
+				error = ENOMEM;
+				break;
+			case KERN_PROTECTION_FAILURE:
+				error = EACCES;
+				break;
+			default:
+				error = EINVAL;
 			}
 			goto bad;
 		}
 	}
-	
-	MALLOC(new_pnode, struct psemnode *, sizeof(struct psemnode), M_SHM, M_WAITOK|M_ZERO);
-	if (new_pnode == NULL) {
-		error = ENOSPC;
-		goto bad;
-	}
+
+	new_pnode = kalloc_type(struct psemnode, Z_WAITOK | Z_ZERO | Z_NOFAIL);
 
 	PSEM_SUBSYS_LOCK();
 	error = psem_cache_search(&pinfo, &nd, &pcache);
 
-	if (error == ENOENT) {
+	if (error == PSEMCACHE_NEGATIVE) {
 		error = EINVAL;
 		goto bad_locked;
-
 	}
-	if (!error) {
-		incache = 0;
-	} else
+
+	if (error == PSEMCACHE_FOUND) {
 		incache = 1;
+	} else {
+		incache = 0;
+	}
 
 	cmode &=  ALLPERMS;
 
-	if (((fmode & (O_CREAT | O_EXCL))==(O_CREAT | O_EXCL)) &&  incache) {
+	if (((fmode & (O_CREAT | O_EXCL)) == (O_CREAT | O_EXCL)) && incache) {
 		/* sem exists and opened O_EXCL */
 #if notyet
 		if (pinfo->psem_flags & PSEM_INDELETE) {
 		}
-#endif 
+#endif
 		AUDIT_ARG(posix_ipc_perm, pinfo->psem_uid,
-			pinfo->psem_gid, pinfo->psem_mode);
+		    pinfo->psem_gid, pinfo->psem_mode);
 		error = EEXIST;
 		goto bad_locked;
 	}
-	if (((fmode & (O_CREAT | O_EXCL))== O_CREAT) &&  incache) {
+	if (((fmode & (O_CREAT | O_EXCL)) == O_CREAT) && incache) {
 		/* As per POSIX, O_CREAT has no effect */
 		fmode &= ~O_CREAT;
 	}
 
-	if ( (fmode & O_CREAT) ) {
+	if ((fmode & O_CREAT)) {
 		/* create a new one (commit the allocation) */
 		pinfo = new_pinfo;
 		pinfo->psem_flags = PSEM_DEFINED | PSEM_INCREATE;
 		pinfo->psem_usecount = 1;
 		pinfo->psem_mode = cmode;
-		pinfo->psem_uid = kauth_cred_getuid(kauth_cred_get());
-		pinfo->psem_gid = kauth_cred_get()->cr_gid;
+		pinfo->psem_uid = kauth_getuid();
+		pinfo->psem_gid = kauth_getgid();
 		bcopy(pnbuf, &pinfo->psem_name[0], PSEMNAMLEN);
-		pinfo->psem_name[PSEMNAMLEN]= 0;
+		pinfo->psem_name[PSEMNAMLEN] = 0;
 		pinfo->psem_flags &= ~PSEM_DEFINED;
 		pinfo->psem_flags |= PSEM_ALLOCATED;
-		pinfo->sem_proc = p;
-		
+		pinfo->psem_creator_pid = proc_getpid(p);
+		pinfo->psem_creator_uniqueid = proc_uniqueid(p);
+
 #if CONFIG_MACF
 		error = mac_posixsem_check_create(kauth_cred_get(), nameptr);
 		if (error) {
@@ -545,26 +554,26 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 			error = ENOENT;
 			goto bad_locked;
 		}
-		if( pinfo->psem_flags & PSEM_INDELETE) {
+		if (pinfo->psem_flags & PSEM_INDELETE) {
 			error = ENOENT;
 			goto bad_locked;
-		}	
+		}
 		AUDIT_ARG(posix_ipc_perm, pinfo->psem_uid,
-			pinfo->psem_gid, pinfo->psem_mode);
+		    pinfo->psem_gid, pinfo->psem_mode);
 #if CONFIG_MACF
 		error = mac_posixsem_check_open(kauth_cred_get(), pinfo);
 		if (error) {
 			goto bad_locked;
 		}
 #endif
-		if ( (error = psem_access(pinfo, fmode, kauth_cred_get())) ) {
+		if ((error = psem_access(pinfo, fmode, kauth_cred_get()))) {
 			goto bad_locked;
 		}
 	}
 
 	if (!incache) {
 		/* if successful, this will consume the pcp */
-		if ( (error = psem_cache_add(pinfo, &nd, pcp)) ) {
+		if ((error = psem_cache_add(pinfo, &nd, pcp))) {
 			goto bad_locked;
 		}
 	}
@@ -578,7 +587,7 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 	 * new . and we must free them.
 	 */
 	if (incache) {
-		FREE(pcp, M_SHM);
+		kfree_type(struct psemcache, pcp);
 		pcp = PSEMCACHE_NULL;
 		if (new_pinfo != PSEMINFO_NULL) {
 			/* return value ignored - we can't _not_ do this */
@@ -586,35 +595,33 @@ sem_open(proc_t p, struct sem_open_args *uap, user_addr_t *retval)
 #if CONFIG_MACF
 			mac_posixsem_label_destroy(new_pinfo);
 #endif
-			FREE(new_pinfo, M_SHM);
+			kfree_type(struct pseminfo, new_pinfo);
 			new_pinfo = PSEMINFO_NULL;
 		}
 	}
 
 	proc_fdlock(p);
 	fp->f_flag = fmode & FMASK;
-	fp->f_type = DTYPE_PSXSEM;
 	fp->f_ops = &psemops;
-	fp->f_data = (caddr_t)new_pnode;
+	fp_set_data(fp, new_pnode);
 	procfdtbl_releasefd(p, indx, NULL);
 	fp_drop(p, indx, fp, 1);
 	proc_fdunlock(p);
 
 	*retval = CAST_USER_ADDR_T(indx);
-	FREE_ZONE(pnbuf, MAXPATHLEN, M_NAMEI);
-	return (0);
+	zfree(ZV_NAMEI, pnbuf);
+	return 0;
 
 bad_locked:
 	PSEM_SUBSYS_UNLOCK();
 bad:
-	if (pcp != PSEMCACHE_NULL)
-		FREE(pcp, M_SHM);
+	kfree_type(struct psemcache, pcp);
 
-	if (new_pnode != PSEMNODE_NULL)
-		FREE(new_pnode, M_SHM);
+	kfree_type(struct psemnode, new_pnode);
 
-	if (fp != NULL)
+	if (fp != NULL) {
 		fp_free(p, indx, fp);
+	}
 
 	if (new_pinfo != PSEMINFO_NULL) {
 		/*
@@ -629,75 +636,85 @@ bad:
 #if CONFIG_MACF
 		mac_posixsem_label_destroy(new_pinfo);
 #endif
-		FREE(new_pinfo, M_SHM);
+		kfree_type(struct pseminfo, new_pinfo);
 	}
 
-	if (pnbuf != NULL)
-		FREE_ZONE(pnbuf, MAXPATHLEN, M_NAMEI);
-	return (error);
+	if (pnbuf != NULL) {
+		zfree(ZV_NAMEI, pnbuf);
+	}
+	return error;
 }
 
 /*
  * XXX This code is repeated in several places
  */
 static int
-psem_access(struct pseminfo *pinfo, int mode, kauth_cred_t cred)
+psem_access(struct pseminfo *pinfo, mode_t mode, kauth_cred_t cred)
 {
-	mode_t mask;
-	int is_member;
+	mode_t mode_req = ((mode & FREAD) ? S_IRUSR : 0) |
+	    ((mode & FWRITE) ? S_IWUSR : 0);
 
 	/* Otherwise, user id 0 always gets access. */
-	if (!suser(cred, NULL))
-		return (0);
-
-	mask = 0;
-
-	/* Otherwise, check the owner. */
-	if (kauth_cred_getuid(cred) == pinfo->psem_uid) {
-		if (mode & FREAD)
-			mask |= S_IRUSR;
-		if (mode & FWRITE)
-			mask |= S_IWUSR;
-		return ((pinfo->psem_mode & mask) == mask ? 0 : EACCES);
+	if (!suser(cred, NULL)) {
+		return 0;
 	}
 
-	/* Otherwise, check the groups. */
-	if (kauth_cred_ismember_gid(cred, pinfo->psem_gid, &is_member) == 0 && is_member) {
-		if (mode & FREAD)
-			mask |= S_IRGRP;
-		if (mode & FWRITE)
-			mask |= S_IWGRP;
-		return ((pinfo->psem_mode & mask) == mask ? 0 : EACCES);
-	}
-
-	/* Otherwise, check everyone else. */
-	if (mode & FREAD)
-		mask |= S_IROTH;
-	if (mode & FWRITE)
-		mask |= S_IWOTH;
-	return ((pinfo->psem_mode & mask) == mask ? 0 : EACCES);
+	return posix_cred_access(cred, pinfo->psem_uid, pinfo->psem_gid, pinfo->psem_mode, mode_req);
 }
 
+static int
+psem_unlink_internal(struct pseminfo *pinfo, struct psemcache *pcache)
+{
+	PSEM_SUBSYS_ASSERT_HELD();
+
+	if (!pinfo || !pcache) {
+		return EINVAL;
+	}
+
+	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED)) == 0) {
+		return EINVAL;
+	}
+
+	if (pinfo->psem_flags & PSEM_INDELETE) {
+		return 0;
+	}
+
+	AUDIT_ARG(posix_ipc_perm, pinfo->psem_uid, pinfo->psem_gid,
+	    pinfo->psem_mode);
+
+	pinfo->psem_flags |= PSEM_INDELETE;
+	pinfo->psem_usecount--;
+
+	if (!pinfo->psem_usecount) {
+		psem_delete(pinfo);
+		kfree_type(struct pseminfo, pinfo);
+	} else {
+		pinfo->psem_flags |= PSEM_REMOVED;
+	}
+
+	psem_cache_delete(pcache);
+	kfree_type(struct psemcache, pcache);
+	return 0;
+}
+
+
 int
 sem_unlink(__unused proc_t p, struct sem_unlink_args *uap, __unused int32_t *retval)
 {
 	size_t i;
-	int error=0;
+	int error = 0;
 	struct psemname nd;
 	struct pseminfo *pinfo;
-	char * pnbuf;
 	char * nameptr;
 	char * cp;
-	size_t pathlen, plen;
-	int incache = 0;
+	char * pnbuf;
+	size_t pathlen;
 	struct psemcache *pcache = PSEMCACHE_NULL;
 
 	pinfo = PSEMINFO_NULL;
 
-	MALLOC_ZONE(pnbuf, caddr_t, MAXPATHLEN, M_NAMEI, M_WAITOK);
-	if (pnbuf == NULL) {
-		return(ENOSPC);		/* XXX non-standard */
-	}
+	pnbuf = zalloc(ZV_NAMEI);
+
 	pathlen = MAXPATHLEN;
 	error = copyinstr(uap->name, pnbuf, MAXPATHLEN, &pathlen);
 	if (error) {
@@ -709,46 +726,38 @@ sem_unlink(__unused proc_t p, struct sem_unlink_args *uap, __unused int32_t *ret
 		goto bad;
 	}
 
+	nameptr = pnbuf;
 
 #ifdef PSXSEM_NAME_RESTRICT
-	nameptr = pnbuf;
 	if (*nameptr == '/') {
 		while (*(nameptr++) == '/') {
-			plen--;
+			pathlen--;
 			error = EINVAL;
 			goto bad;
 		}
-        } else {
+	} else {
 		error = EINVAL;
 		goto bad;
 	}
 #endif /* PSXSEM_NAME_RESTRICT */
 
-	plen = pathlen;
-	nameptr = pnbuf;
 	nd.psem_nameptr = nameptr;
-	nd.psem_namelen = plen;
-	nd. psem_hash =0;
+	nd.psem_namelen = pathlen;
+	nd.psem_hash = 0;
 
-        for (cp = nameptr, i=1; *cp != 0 && i <= plen; i++, cp++) {
-               nd.psem_hash += (unsigned char)*cp * i;
+	for (cp = nameptr, i = 1; *cp != 0 && i <= pathlen; i++, cp++) {
+		nd.psem_hash += (unsigned char)*cp * i;
 	}
 
 	PSEM_SUBSYS_LOCK();
 	error = psem_cache_search(&pinfo, &nd, &pcache);
 
-	if (error == ENOENT) {
+	if (error != PSEMCACHE_FOUND) {
 		PSEM_SUBSYS_UNLOCK();
-		error = EINVAL;
+		error = ENOENT;
 		goto bad;
-
 	}
-	if (!error) {
-		PSEM_SUBSYS_UNLOCK();
-		error = EINVAL;
-		goto bad;
-	} else
-		incache = 1;
+
 #if CONFIG_MACF
 	error = mac_posixsem_check_unlink(kauth_cred_get(), pinfo, nameptr);
 	if (error) {
@@ -756,99 +765,70 @@ sem_unlink(__unused proc_t p, struct sem_unlink_args *uap, __unused int32_t *ret
 		goto bad;
 	}
 #endif
-	if ( (error = psem_access(pinfo, pinfo->psem_mode, kauth_cred_get())) ) {
+	if ((error = psem_access(pinfo, pinfo->psem_mode, kauth_cred_get()))) {
 		PSEM_SUBSYS_UNLOCK();
 		goto bad;
 	}
 
-	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED))==0) {
-		PSEM_SUBSYS_UNLOCK();
-		error = EINVAL;
-		goto bad;
-	}
-
-	if ( (pinfo->psem_flags & PSEM_INDELETE) ) {
-		PSEM_SUBSYS_UNLOCK();
-		error = 0;
-		goto bad;
-	}
-
-	AUDIT_ARG(posix_ipc_perm, pinfo->psem_uid, pinfo->psem_gid,
-		  pinfo->psem_mode);
-
-	pinfo->psem_flags |= PSEM_INDELETE;
-	pinfo->psem_usecount--;
-
-	if (!pinfo->psem_usecount) {
-		psem_delete(pinfo);
-		FREE(pinfo,M_SHM);
-	} else
-		pinfo->psem_flags |= PSEM_REMOVED;
-
-	psem_cache_delete(pcache);
+	error = psem_unlink_internal(pinfo, pcache);
 	PSEM_SUBSYS_UNLOCK();
-	FREE(pcache, M_SHM);
-	error = 0;
+
 bad:
-	FREE_ZONE(pnbuf, MAXPATHLEN, M_NAMEI);
-	return (error);
+	zfree(ZV_NAMEI, pnbuf);
+	return error;
 }
 
 int
 sem_close(proc_t p, struct sem_close_args *uap, __unused int32_t *retval)
 {
-	int fd = CAST_DOWN_EXPLICIT(int,uap->sem);
+	int fd = CAST_DOWN_EXPLICIT(int, uap->sem);
 	struct fileproc *fp;
-	int error = 0;
 
 	AUDIT_ARG(fd, fd); /* XXX This seems wrong; uap->sem is a pointer */
 
 	proc_fdlock(p);
-	error = fp_lookup(p,fd, &fp, 1);
-	if (error) {
+	if ((fp = fp_get_noref_locked(p, fd)) == NULL) {
 		proc_fdunlock(p);
-		return(error);
+		return EBADF;
 	}
-	fileproc_drain(p, fp);
-	fdrelse(p, fd);
-	error = closef_locked(fp, fp->f_fglob, p);
-	FREE_ZONE(fp, sizeof *fp, M_FILEPROC);
-	proc_fdunlock(p);
-	return(error);
+	if (FILEGLOB_DTYPE(fp->fp_glob) != DTYPE_PSXSEM) {
+		proc_fdunlock(p);
+		return EBADF;
+	}
+	return fp_close_and_unlock(p, fd, fp, 0);
 }
 
 int
 sem_wait(proc_t p, struct sem_wait_args *uap, int32_t *retval)
 {
 	__pthread_testcancel(1);
-	return(sem_wait_nocancel(p, (struct sem_wait_nocancel_args *)uap, retval));
+	return sem_wait_nocancel(p, (struct sem_wait_nocancel_args *)uap, retval);
 }
 
 int
 sem_wait_nocancel(proc_t p, struct sem_wait_nocancel_args *uap, __unused int32_t *retval)
 {
-	int fd = CAST_DOWN_EXPLICIT(int,uap->sem);
+	int fd = CAST_DOWN_EXPLICIT(int, uap->sem);
 	struct fileproc *fp;
 	struct pseminfo * pinfo;
-	struct psemnode * pnode ;
+	struct psemnode * pnode;
 	kern_return_t kret;
 	int error;
 
-	error = fp_getfpsem(p, fd, &fp, &pnode);
-	if (error)
-		return (error);
-	if (((pnode = (struct psemnode *)fp->f_data)) == PSEMNODE_NULL )  {
-		error = EINVAL;
-		goto out;
+	error = fp_get_ftype(p, fd, DTYPE_PSXSEM, EBADF, &fp);
+	if (error) {
+		return error;
 	}
+	pnode = (struct psemnode *)fp_get_data(fp);
+
 	PSEM_SUBSYS_LOCK();
 	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL) {
 		PSEM_SUBSYS_UNLOCK();
 		error = EINVAL;
 		goto out;
 	}
-	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED)) 
-			!= PSEM_ALLOCATED) {
+	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED))
+	    != PSEM_ALLOCATED) {
 		PSEM_SUBSYS_UNLOCK();
 		error = EINVAL;
 		goto out;
@@ -880,36 +860,34 @@ sem_wait_nocancel(proc_t p, struct sem_wait_nocancel_args *uap, __unused int32_t
 	}
 out:
 	fp_drop(p, fd, fp, 0);
-	return(error);
-
+	return error;
 }
 
 int
 sem_trywait(proc_t p, struct sem_trywait_args *uap, __unused int32_t *retval)
 {
-	int fd = CAST_DOWN_EXPLICIT(int,uap->sem);
+	int fd = CAST_DOWN_EXPLICIT(int, uap->sem);
 	struct fileproc *fp;
 	struct pseminfo * pinfo;
-	struct psemnode * pnode ;
+	struct psemnode * pnode;
 	kern_return_t kret;
 	mach_timespec_t wait_time;
 	int error;
-	
-	error = fp_getfpsem(p, fd, &fp, &pnode);
-	if (error)
-		return (error);
-	if (((pnode = (struct psemnode *)fp->f_data)) == PSEMNODE_NULL )  {
-		error = EINVAL;
-		goto out;
+
+	error = fp_get_ftype(p, fd, DTYPE_PSXSEM, EBADF, &fp);
+	if (error) {
+		return error;
 	}
+	pnode = (struct psemnode *)fp_get_data(fp);
+
 	PSEM_SUBSYS_LOCK();
 	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL) {
 		PSEM_SUBSYS_UNLOCK();
 		error = EINVAL;
 		goto out;
 	}
-	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED)) 
-			!= PSEM_ALLOCATED) {
+	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED))
+	    != PSEM_ALLOCATED) {
 		PSEM_SUBSYS_UNLOCK();
 		error = EINVAL;
 		goto out;
@@ -946,34 +924,33 @@ sem_trywait(proc_t p, struct sem_trywait_args *uap, __unused int32_t *retval)
 	}
 out:
 	fp_drop(p, fd, fp, 0);
-	return(error);
+	return error;
 }
 
 int
 sem_post(proc_t p, struct sem_post_args *uap, __unused int32_t *retval)
 {
-	int fd = CAST_DOWN_EXPLICIT(int,uap->sem);
+	int fd = CAST_DOWN_EXPLICIT(int, uap->sem);
 	struct fileproc *fp;
 	struct pseminfo * pinfo;
-	struct psemnode * pnode ;
+	struct psemnode * pnode;
 	kern_return_t kret;
 	int error;
 
-	error = fp_getfpsem(p, fd, &fp, &pnode);
-	if (error)
-		return (error);
-	if (((pnode = (struct psemnode *)fp->f_data)) == PSEMNODE_NULL )  {
-		error = EINVAL;
-		goto out;
+	error = fp_get_ftype(p, fd, DTYPE_PSXSEM, EBADF, &fp);
+	if (error) {
+		return error;
 	}
+	pnode = (struct psemnode *)fp_get_data(fp);
+
 	PSEM_SUBSYS_LOCK();
 	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL) {
 		PSEM_SUBSYS_UNLOCK();
 		error = EINVAL;
 		goto out;
 	}
-	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED)) 
-			!= PSEM_ALLOCATED) {
+	if ((pinfo->psem_flags & (PSEM_DEFINED | PSEM_ALLOCATED))
+	    != PSEM_ALLOCATED) {
 		PSEM_SUBSYS_UNLOCK();
 		error = EINVAL;
 		goto out;
@@ -1005,78 +982,56 @@ sem_post(proc_t p, struct sem_post_args *uap, __unused int32_t *retval)
 	}
 out:
 	fp_drop(p, fd, fp, 0);
-	return(error);
-}
-
-int
-sem_init(__unused proc_t p, __unused struct sem_init_args *uap, __unused int32_t *retval)
-{
-	return(ENOSYS);
-}
-
-int
-sem_destroy(__unused proc_t p, __unused struct sem_destroy_args *uap, __unused int32_t *retval)
-{
-	return(ENOSYS);
-}
-
-int
-sem_getvalue(__unused proc_t p, __unused struct sem_getvalue_args *uap, __unused int32_t *retval)
-{
-	return(ENOSYS);
+	return error;
 }
 
 static int
-psem_close(struct psemnode *pnode, __unused int flags)
+psem_close(struct psemnode *pnode)
 {
-	int error=0;
+	int error = 0;
 	struct pseminfo *pinfo;
 
 	PSEM_SUBSYS_LOCK();
-	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL){
+	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL) {
 		PSEM_SUBSYS_UNLOCK();
-		return(EINVAL);
+		return EINVAL;
 	}
 
 	if ((pinfo->psem_flags & PSEM_ALLOCATED) != PSEM_ALLOCATED) {
 		PSEM_SUBSYS_UNLOCK();
-		return(EINVAL);
+		return EINVAL;
 	}
 #if DIAGNOSTIC
-	if(!pinfo->psem_usecount) {
+	if (!pinfo->psem_usecount) {
 		kprintf("negative usecount in psem_close\n");
 	}
 #endif /* DIAGNOSTIC */
 	pinfo->psem_usecount--;
 
- 	if ((pinfo->psem_flags & PSEM_REMOVED) && !pinfo->psem_usecount) {
+	if ((pinfo->psem_flags & PSEM_REMOVED) && !pinfo->psem_usecount) {
 		PSEM_SUBSYS_UNLOCK();
 		/* lock dropped as only semaphore is destroyed here */
 		error = psem_delete(pinfo);
-		FREE(pinfo,M_SHM);
+		kfree_type(struct pseminfo, pinfo);
 	} else {
 		PSEM_SUBSYS_UNLOCK();
 	}
 	/* subsystem lock is dropped when we get here */
-	FREE(pnode, M_SHM);
-	return (error);
+	kfree_type(struct psemnode, pnode);
+	return error;
 }
 
 static int
 psem_closefile(struct fileglob *fg, __unused vfs_context_t ctx)
 {
-	int error;
-
 	/*
 	 * Not locked as psem_close is called only from here and is locked
 	 * properly
 	 */
-	error =  psem_close(((struct psemnode *)fg->fg_data), fg->fg_flag);
-
-	return(error);
+	return psem_close((struct psemnode *)fg_get_data(fg));
 }
 
-static int 
+static int
 psem_delete(struct pseminfo * pinfo)
 {
 	kern_return_t kret;
@@ -1089,52 +1044,17 @@ psem_delete(struct pseminfo * pinfo)
 	switch (kret) {
 	case KERN_INVALID_ADDRESS:
 	case KERN_PROTECTION_FAILURE:
-		return (EINVAL);
+		return EINVAL;
 	case KERN_ABORTED:
 	case KERN_OPERATION_TIMED_OUT:
-		return (EINTR);
+		return EINTR;
 	case KERN_SUCCESS:
-		return(0);
+		return 0;
 	default:
-		return (EINVAL);
+		return EINVAL;
 	}
 }
 
-static int
-psem_read(__unused struct fileproc *fp, __unused struct uio *uio, 
-		  __unused int flags, __unused vfs_context_t ctx)
-{
-	return(ENOTSUP);
-}
-
-static int
-psem_write(__unused struct fileproc *fp, __unused struct uio *uio, 
-		   __unused int flags, __unused vfs_context_t ctx)
-{
-	return(ENOTSUP);
-}
-
-static int
-psem_ioctl(__unused struct fileproc *fp, __unused u_long com, 
-			__unused caddr_t data, __unused vfs_context_t ctx)
-{
-	return(ENOTSUP);
-}
-
-static int
-psem_select(__unused struct fileproc *fp, __unused int which, 
-			__unused void *wql, __unused vfs_context_t ctx)
-{
-	return(ENOTSUP);
-}
-
-static int
-psem_kqfilter(__unused struct fileproc *fp, __unused struct knote *kn, 
-				__unused vfs_context_t ctx)
-{
-	return (ENOTSUP);
-}
-
 int
 fill_pseminfo(struct psemnode *pnode, struct psem_info * info)
 {
@@ -1142,29 +1062,29 @@ fill_pseminfo(struct psemnode *pnode, struct psem_info * info)
 	struct vinfo_stat  *sb;
 
 	PSEM_SUBSYS_LOCK();
-	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL){
+	if ((pinfo = pnode->pinfo) == PSEMINFO_NULL) {
 		PSEM_SUBSYS_UNLOCK();
-		return(EINVAL);
+		return EINVAL;
 	}
 
 #if 0
 	if ((pinfo->psem_flags & PSEM_ALLOCATED) != PSEM_ALLOCATED) {
 		PSEM_SUBSYS_UNLOCK();
-		return(EINVAL);
+		return EINVAL;
 	}
 #endif
 
 	sb = &info->psem_stat;
 	bzero(sb, sizeof(struct vinfo_stat));
 
-    	sb->vst_mode = pinfo->psem_mode;
-    	sb->vst_uid = pinfo->psem_uid;
-    	sb->vst_gid = pinfo->psem_gid;
-    	sb->vst_size = pinfo->psem_usecount;
-	bcopy(&pinfo->psem_name[0], &info->psem_name[0], PSEMNAMLEN+1);
+	sb->vst_mode = pinfo->psem_mode;
+	sb->vst_uid = pinfo->psem_uid;
+	sb->vst_gid = pinfo->psem_gid;
+	sb->vst_size = pinfo->psem_usecount;
+	bcopy(&pinfo->psem_name[0], &info->psem_name[0], PSEMNAMLEN + 1);
 
 	PSEM_SUBSYS_UNLOCK();
-	return(0);
+	return 0;
 }
 
 #if CONFIG_MACF
@@ -1175,15 +1095,16 @@ psem_label_associate(struct fileproc *fp, struct vnode *vp, vfs_context_t ctx)
 	struct pseminfo *psem;
 
 	PSEM_SUBSYS_LOCK();
-	pnode = (struct psemnode *)fp->f_fglob->fg_data;
+	pnode = (struct psemnode *)fp_get_data(fp);
 	if (pnode != NULL) {
 		psem = pnode->pinfo;
-		if (psem != NULL)
+		if (psem != NULL) {
 			mac_posixsem_vnode_label_associate(
-				vfs_context_ucred(ctx), psem, psem->psem_label,
-				vp, vp->v_label);
+				vfs_context_ucred(ctx), psem,
+				mac_posixsem_label(psem),
+				vp, mac_vnode_label(vp));
+		}
 	}
 	PSEM_SUBSYS_UNLOCK();
 }
 #endif
-      
