@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2000-2008 Apple Inc. All rights reserved.
+ * Copyright (c) 2000-2020 Apple Inc. All rights reserved.
  *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_START@
- * 
+ *
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -11,10 +11,10 @@
  * unlawful or unlicensed copies of an Apple operating system, or to
  * circumvent, violate, or enable the circumvention or violation of, any
  * terms of an Apple operating system software license agreement.
- * 
+ *
  * Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this file.
- * 
+ *
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -22,7 +22,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- * 
+ *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_END@
  */
 
@@ -94,7 +94,7 @@
  * ICMPv6:
  * - Redirects cannot be used due to the lack of link-local address.
  *
- * stf interface does not have, and will not need, a link-local address.  
+ * stf interface does not have, and will not need, a link-local address.
  * It seems to have no real benefit and does not help the above symptoms much.
  * Even if we assign link-locals to interface, we cannot really
  * use link-local unicast/multicast on top of 6to4 cloud (since there's no
@@ -120,6 +120,8 @@
 
 #include <sys/malloc.h>
 
+#include <kern/locks.h>
+
 #include <net/if.h>
 #include <net/route.h>
 #include <net/if_types.h>
@@ -144,50 +146,49 @@
 
 #include <net/bpf.h>
 
-#if CONFIG_MACF_NET
-#include <security/mac_framework.h>
-#endif
+#define GET_V4(x) ((const struct in_addr *)(const void *)(&(x)->s6_addr16[1]))
 
-#define IN6_IS_ADDR_6TO4(x)	(ntohs((x)->s6_addr16[0]) == 0x2002)
-#define GET_V4(x)	((const struct in_addr *)(&(x)->s6_addr16[1]))
+static LCK_GRP_DECLARE(stf_mtx_grp, "stf");
 
 struct stf_softc {
-	ifnet_t				sc_if;	   /* common area */
-	u_int32_t				sc_protocol_family; /* dlil protocol attached */
+	ifnet_t                         sc_if;     /* common area */
+	u_int32_t                               sc_protocol_family; /* dlil protocol attached */
 	union {
 		struct route  __sc_ro4;
 		struct route_in6 __sc_ro6; /* just for safety */
 	} __sc_ro46;
-#define sc_ro	__sc_ro46.__sc_ro4
+#define sc_ro   __sc_ro46.__sc_ro4
+	decl_lck_mtx_data(, sc_ro_mtx);
 	const struct encaptab *encap_cookie;
-	bpf_tap_mode		tap_mode;
-	bpf_packet_func		tap_callback;
+	bpf_tap_mode            tap_mode;
+	bpf_packet_func         tap_callback;
 };
 
-void stfattach (void);
+void stfattach(void);
 
 static int ip_stf_ttl = 40;
 
 static void in_stf_input(struct mbuf *, int);
-extern  struct domain inetdomain;
-struct protosw in_stf_protosw =
-{ SOCK_RAW,	&inetdomain,	IPPROTO_IPV6,	PR_ATOMIC|PR_ADDR,
-  in_stf_input, NULL,	NULL,		rip_ctloutput,
-  NULL,
-  NULL,		NULL,	NULL,	NULL,
-  NULL,
-  &rip_usrreqs,
-  NULL,		rip_unlock,	NULL, {NULL, NULL}, NULL, {0}
+
+static struct protosw in_stf_protosw =
+{
+	.pr_type =              SOCK_RAW,
+	.pr_protocol =          IPPROTO_IPV6,
+	.pr_flags =             PR_ATOMIC | PR_ADDR,
+	.pr_input =             in_stf_input,
+	.pr_ctloutput =         rip_ctloutput,
+	.pr_usrreqs =           &rip_usrreqs,
+	.pr_unlock =            rip_unlock,
 };
 
 static int stf_encapcheck(const struct mbuf *, int, int, void *);
 static struct in6_ifaddr *stf_getsrcifa6(struct ifnet *);
 int stf_pre_output(struct ifnet *, protocol_family_t, struct mbuf **,
-	const struct sockaddr *, void *, char *, char *);
+    const struct sockaddr *, void *, char *, char *);
 static int stf_checkaddr4(struct stf_softc *, const struct in_addr *,
-	struct ifnet *);
+    struct ifnet *);
 static int stf_checkaddr6(struct stf_softc *, struct in6_addr *,
-	struct ifnet *);
+    struct ifnet *);
 static void stf_rtrequest(int, struct rtentry *, struct sockaddr *);
 static errno_t stf_ioctl(ifnet_t ifp, u_long cmd, void *data);
 static errno_t stf_output(ifnet_t ifp, mbuf_t m);
@@ -197,78 +198,82 @@ static errno_t stf_output(ifnet_t ifp, mbuf_t m);
  */
 static errno_t
 stf_media_input(
-	__unused ifnet_t	ifp,
-	protocol_family_t	protocol_family,
-	mbuf_t				m,
-	__unused char		*frame_header)
+	__unused ifnet_t        ifp,
+	protocol_family_t       protocol_family,
+	mbuf_t                          m,
+	__unused char           *frame_header)
 {
-	proto_input(protocol_family, m);
+	if (proto_input(protocol_family, m) != 0) {
+		m_freem(m);
+	}
 
-	return (0);
+	return 0;
 }
 
 
 
 static errno_t
 stf_add_proto(
-	ifnet_t									ifp,
-	protocol_family_t						protocol_family,
-	__unused const struct ifnet_demux_desc	*demux_array,
-	__unused u_int32_t						demux_count)
+	ifnet_t                                                                 ifp,
+	protocol_family_t                                               protocol_family,
+	__unused const struct ifnet_demux_desc  *demux_array,
+	__unused u_int32_t                                              demux_count)
 {
 	/* Only one protocol may be attached at a time */
 	struct stf_softc* stf = ifnet_softc(ifp);
-	if (stf->sc_protocol_family == 0)
+	if (stf->sc_protocol_family == 0) {
 		stf->sc_protocol_family = protocol_family;
-	else {
+	} else {
 		printf("stf_add_proto: stf already has a proto\n");
 		return EBUSY;
 	}
-	
+
 	return 0;
 }
 
 static errno_t
 stf_del_proto(
-	ifnet_t				ifp,
-	protocol_family_t	protocol_family)
+	ifnet_t                         ifp,
+	protocol_family_t       protocol_family)
 {
-	if (((struct stf_softc*)ifnet_softc(ifp))->sc_protocol_family == protocol_family)
+	if (((struct stf_softc*)ifnet_softc(ifp))->sc_protocol_family == protocol_family) {
 		((struct stf_softc*)ifnet_softc(ifp))->sc_protocol_family = 0;
-	
+	}
+
 	return 0;
 }
 
 static errno_t
 stf_attach_inet6(
-	ifnet_t				ifp,
-	protocol_family_t	protocol_family)
+	ifnet_t                         ifp,
+	protocol_family_t       protocol_family)
 {
-    struct ifnet_attach_proto_param	reg;
-    errno_t							stat;
-    
-    if (protocol_family != PF_INET6)
-    	return EPROTONOSUPPORT;
+	struct ifnet_attach_proto_param     reg;
+	errno_t                                                     stat;
+
+	if (protocol_family != PF_INET6) {
+		return EPROTONOSUPPORT;
+	}
 
 	bzero(&reg, sizeof(reg));
-    reg.input = stf_media_input;
-    reg.pre_output = stf_pre_output;
+	reg.input = stf_media_input;
+	reg.pre_output = stf_pre_output;
 
-    stat = ifnet_attach_protocol(ifp, protocol_family, &reg);
-    if (stat && stat != EEXIST) {
-        printf("stf_attach_proto_family can't attach interface fam=%d\n",
-        	   protocol_family);
-    }
+	stat = ifnet_attach_protocol(ifp, protocol_family, &reg);
+	if (stat && stat != EEXIST) {
+		printf("stf_attach_proto_family can't attach interface fam=%d\n",
+		    protocol_family);
+	}
 
-    return stat;
+	return stat;
 }
 
 static errno_t
 stf_demux(
-	ifnet_t					ifp,
-	__unused mbuf_t			m,
-	__unused char			*frame_ptr,
-	protocol_family_t		*protocol_family)
+	ifnet_t                                 ifp,
+	__unused mbuf_t                 m,
+	__unused char                   *frame_ptr,
+	protocol_family_t               *protocol_family)
 {
 	struct stf_softc* stf = ifnet_softc(ifp);
 	*protocol_family = stf->sc_protocol_family;
@@ -277,15 +282,15 @@ stf_demux(
 
 static errno_t
 stf_set_bpf_tap(
-	ifnet_t			ifp,
-	bpf_tap_mode	mode,
-	bpf_packet_func	callback)
+	ifnet_t                 ifp,
+	bpf_tap_mode    mode,
+	bpf_packet_func callback)
 {
-	struct stf_softc	*sc = ifnet_softc(ifp);
-	
+	struct stf_softc        *sc = ifnet_softc(ifp);
+
 	sc->tap_mode = mode;
 	sc->tap_callback = callback;
-	
+
 	return 0;
 }
 
@@ -295,31 +300,29 @@ stfattach(void)
 	struct stf_softc *sc;
 	int error;
 	const struct encaptab *p;
-	struct ifnet_init_params	stf_init;
+	struct ifnet_init_eparams       stf_init;
 
 	error = proto_register_plumber(PF_INET6, APPLE_IF_FAM_STF,
-								   stf_attach_inet6, NULL);
-	if (error != 0)
+	    stf_attach_inet6, NULL);
+	if (error != 0) {
 		printf("proto_register_plumber failed for AF_INET6 error=%d\n", error);
-
-	sc = _MALLOC(sizeof(struct stf_softc), M_DEVBUF, M_WAITOK);
-	if (sc == 0) {
-		printf("stf softc attach failed\n" );
-		return;
 	}
-	
-	bzero(sc, sizeof(*sc));
-	
+
+	sc = kalloc_type(struct stf_softc, Z_WAITOK_ZERO_NOFAIL);
+	lck_mtx_init(&sc->sc_ro_mtx, &stf_mtx_grp, LCK_ATTR_NULL);
+
 	p = encap_attach_func(AF_INET, IPPROTO_IPV6, stf_encapcheck,
 	    &in_stf_protosw, sc);
 	if (p == NULL) {
 		printf("sftattach encap_attach_func failed\n");
-		FREE(sc, M_DEVBUF);
-		return;
+		goto free_sc;
 	}
 	sc->encap_cookie = p;
-	
+
 	bzero(&stf_init, sizeof(stf_init));
+	stf_init.ver = IFNET_INIT_CURRENT_VERSION;
+	stf_init.len = sizeof(stf_init);
+	stf_init.flags = IFNET_INIT_LEGACY;
 	stf_init.name = "stf";
 	stf_init.unit = 0;
 	stf_init.type = IFT_STF;
@@ -331,13 +334,12 @@ stfattach(void)
 	stf_init.softc = sc;
 	stf_init.ioctl = stf_ioctl;
 	stf_init.set_bpf_tap = stf_set_bpf_tap;
-	
-	error = ifnet_allocate(&stf_init, &sc->sc_if);
+
+	error = ifnet_allocate_extended(&stf_init, &sc->sc_if);
 	if (error != 0) {
 		printf("stfattach, ifnet_allocate failed - %d\n", error);
 		encap_detach(sc->encap_cookie);
-		FREE(sc, M_DEVBUF);
-		return;
+		goto free_sc;
 	}
 	ifnet_set_mtu(sc->sc_if, IPV6_MMTU);
 	ifnet_set_flags(sc->sc_if, 0, 0xffff); /* clear all flags */
@@ -346,22 +348,21 @@ stfattach(void)
 	ifnet_set_flags(sc->sc_if, IFF_LINK2, IFF_LINK2);
 #endif
 
-#if CONFIG_MACF_NET
-	mac_ifnet_label_init(&sc->sc_if);
-#endif
-	
 	error = ifnet_attach(sc->sc_if, NULL);
 	if (error != 0) {
 		printf("stfattach: ifnet_attach returned error=%d\n", error);
 		encap_detach(sc->encap_cookie);
 		ifnet_release(sc->sc_if);
-		FREE(sc, M_DEVBUF);
-		return;
+		goto free_sc;
 	}
-	
+
 	bpfattach(sc->sc_if, DLT_NULL, sizeof(u_int));
-	
+
 	return;
+
+free_sc:
+	lck_mtx_destroy(&sc->sc_ro_mtx, &stf_mtx_grp);
+	kfree_type(struct stf_softc, sc);
 }
 
 static int
@@ -377,36 +378,44 @@ stf_encapcheck(
 	struct in_addr a, b;
 
 	sc = (struct stf_softc *)arg;
-	if (sc == NULL)
+	if (sc == NULL) {
 		return 0;
+	}
 
-	if ((ifnet_flags(sc->sc_if) & IFF_UP) == 0)
+	if ((ifnet_flags(sc->sc_if) & IFF_UP) == 0) {
 		return 0;
+	}
 
 	/* IFF_LINK0 means "no decapsulation" */
-	if ((ifnet_flags(sc->sc_if) & IFF_LINK0) != 0)
+	if ((ifnet_flags(sc->sc_if) & IFF_LINK0) != 0) {
 		return 0;
+	}
 
-	if (proto != IPPROTO_IPV6)
+	if (proto != IPPROTO_IPV6) {
 		return 0;
+	}
 
 	mbuf_copydata((struct mbuf *)(size_t)m, 0, sizeof(ip), &ip);
 
-	if (ip.ip_v != 4)
+	if (ip.ip_v != 4) {
 		return 0;
+	}
 
 	ia6 = stf_getsrcifa6(sc->sc_if);
-	if (ia6 == NULL)
+	if (ia6 == NULL) {
 		return 0;
+	}
 
 	/*
 	 * check if IPv4 dst matches the IPv4 address derived from the
 	 * local 6to4 address.
 	 * success on: dst = 10.1.1.1, ia6->ia_addr = 2002:0a01:0101:...
 	 */
+	IFA_LOCK(&ia6->ia_ifa);
 	if (bcmp(GET_V4(&ia6->ia_addr.sin6_addr), &ip.ip_dst,
 	    sizeof(ip.ip_dst)) != 0) {
-		ifafree(&ia6->ia_ifa);
+		IFA_UNLOCK(&ia6->ia_ifa);
+		IFA_REMREF(&ia6->ia_ifa);
 		return 0;
 	}
 	/*
@@ -421,11 +430,13 @@ stf_encapcheck(
 	b = ip.ip_src;
 	b.s_addr &= GET_V4(&ia6->ia_prefixmask.sin6_addr)->s_addr;
 	if (a.s_addr != b.s_addr) {
-		ifafree(&ia6->ia_ifa);
+		IFA_UNLOCK(&ia6->ia_ifa);
+		IFA_REMREF(&ia6->ia_ifa);
 		return 0;
 	}
 	/* stf interface makes single side match only */
-	ifafree(&ia6->ia_ifa);
+	IFA_UNLOCK(&ia6->ia_ifa);
+	IFA_REMREF(&ia6->ia_ifa);
 	return 32;
 }
 
@@ -438,32 +449,40 @@ stf_getsrcifa6(struct ifnet *ifp)
 	struct in_addr in;
 
 	ifnet_lock_shared(ifp);
-	for (ia = ifp->if_addrlist.tqh_first;
-	     ia;
-	     ia = ia->ifa_list.tqe_next)
-	{
-		if (ia->ifa_addr == NULL)
+	for (ia = ifp->if_addrlist.tqh_first; ia; ia = ia->ifa_list.tqe_next) {
+		IFA_LOCK(ia);
+		if (ia->ifa_addr == NULL) {
+			IFA_UNLOCK(ia);
 			continue;
-		if (ia->ifa_addr->sa_family != AF_INET6)
-			continue;
-		sin6 = (struct sockaddr_in6 *)ia->ifa_addr;
-		if (!IN6_IS_ADDR_6TO4(&sin6->sin6_addr))
-			continue;
-
-		bcopy(GET_V4(&sin6->sin6_addr), &in, sizeof(in));
-		lck_rw_lock_shared(in_ifaddr_rwlock);
-		for (ia4 = TAILQ_FIRST(&in_ifaddrhead);
-		     ia4;
-		     ia4 = TAILQ_NEXT(ia4, ia_link))
-		{
-			if (ia4->ia_addr.sin_addr.s_addr == in.s_addr)
-				break;
 		}
-		lck_rw_done(in_ifaddr_rwlock);
-		if (ia4 == NULL)
+		if (ia->ifa_addr->sa_family != AF_INET6) {
+			IFA_UNLOCK(ia);
 			continue;
+		}
+		sin6 = (struct sockaddr_in6 *)(void *)ia->ifa_addr;
+		if (!IN6_IS_ADDR_6TO4(&sin6->sin6_addr)) {
+			IFA_UNLOCK(ia);
+			continue;
+		}
+		bcopy(GET_V4(&sin6->sin6_addr), &in, sizeof(in));
+		IFA_UNLOCK(ia);
+		lck_rw_lock_shared(&in_ifaddr_rwlock);
+		for (ia4 = TAILQ_FIRST(&in_ifaddrhead);
+		    ia4;
+		    ia4 = TAILQ_NEXT(ia4, ia_link)) {
+			IFA_LOCK(&ia4->ia_ifa);
+			if (ia4->ia_addr.sin_addr.s_addr == in.s_addr) {
+				IFA_UNLOCK(&ia4->ia_ifa);
+				break;
+			}
+			IFA_UNLOCK(&ia4->ia_ifa);
+		}
+		lck_rw_done(&in_ifaddr_rwlock);
+		if (ia4 == NULL) {
+			continue;
+		}
 
-		ifaref(ia);
+		IFA_ADDREF(ia);         /* for caller */
 		ifnet_lock_done(ifp);
 		return (struct in6_ifaddr *)ia;
 	}
@@ -474,10 +493,10 @@ stf_getsrcifa6(struct ifnet *ifp)
 
 int
 stf_pre_output(
-	struct ifnet	*ifp,
+	struct ifnet    *ifp,
 	__unused protocol_family_t  protocol_family,
-	struct mbuf	**m0,
-	const struct sockaddr	*dst,
+	struct mbuf     **m0,
+	const struct sockaddr   *dst,
 	__unused void *route,
 	__unused char *desk_linkaddr,
 	__unused char *frame_type)
@@ -490,11 +509,18 @@ stf_pre_output(
 	struct ip *ip;
 	struct ip6_hdr *ip6;
 	struct in6_ifaddr *ia6;
-	struct sockaddr_in 	*dst4;
-	errno_t				result = 0;
+	struct sockaddr_in      *dst4;
+	struct ip_out_args ipoa;
+	errno_t result = 0;
+
+	bzero(&ipoa, sizeof(ipoa));
+	ipoa.ipoa_boundif = IFSCOPE_NONE;
+	ipoa.ipoa_flags = IPOAF_SELECT_SRCIF;
+	ipoa.ipoa_sotc = SO_TC_UNSPEC;
+	ipoa.ipoa_netsvctype = _NET_SERVICE_TYPE_UNSPEC;
 
 	sc = ifnet_softc(ifp);
-	dst6 = (const struct sockaddr_in6 *)dst;
+	dst6 = (const struct sockaddr_in6 *)(const void *)dst;
 
 	/* just in case */
 	if ((ifnet_flags(ifp) & IFF_UP) == 0) {
@@ -516,9 +542,10 @@ stf_pre_output(
 		m = m_pullup(m, sizeof(*ip6));
 		if (!m) {
 			*m0 = NULL; /* makes sure this won't be double freed */
-			ifafree(&ia6->ia_ifa);
+			IFA_REMREF(&ia6->ia_ifa);
 			return ENOBUFS;
 		}
+		*m0 = m;
 	}
 	ip6 = mtod(m, struct ip6_hdr *);
 	tos = (ntohl(ip6->ip6_flow) >> 20) & 0xff;
@@ -527,92 +554,90 @@ stf_pre_output(
 	 * Pickup the right outer dst addr from the list of candidates.
 	 * ip6_dst has priority as it may be able to give us shorter IPv4 hops.
 	 */
-	if (IN6_IS_ADDR_6TO4(&ip6->ip6_dst))
+	if (IN6_IS_ADDR_6TO4(&ip6->ip6_dst)) {
 		in4 = GET_V4(&ip6->ip6_dst);
-	else if (IN6_IS_ADDR_6TO4(&dst6->sin6_addr))
+	} else if (IN6_IS_ADDR_6TO4(&dst6->sin6_addr)) {
 		in4 = GET_V4(&dst6->sin6_addr);
-	else {
-		ifafree(&ia6->ia_ifa);
+	} else {
+		IFA_REMREF(&ia6->ia_ifa);
 		return ENETUNREACH;
 	}
 
 	if (ifp->if_bpf) {
 		/* We need to prepend the address family as a four byte field. */
 		u_int32_t af = AF_INET6;
-		
+
 		bpf_tap_out(ifp, 0, m, &af, sizeof(af));
 	}
 
-	M_PREPEND(m, sizeof(struct ip), M_DONTWAIT);
-	if (m && mbuf_len(m) < sizeof(struct ip))
+	M_PREPEND(m, sizeof(struct ip), M_DONTWAIT, 1);
+	if (m && mbuf_len(m) < sizeof(struct ip)) {
 		m = m_pullup(m, sizeof(struct ip));
+	}
 	if (m == NULL) {
-		*m0 = NULL; 
-		ifafree(&ia6->ia_ifa);
+		*m0 = NULL;
+		IFA_REMREF(&ia6->ia_ifa);
 		return ENOBUFS;
 	}
+
+	*m0 = m;
 	ip = mtod(m, struct ip *);
 
 	bzero(ip, sizeof(*ip));
 
+	IFA_LOCK_SPIN(&ia6->ia_ifa);
 	bcopy(GET_V4(&((struct sockaddr_in6 *)&ia6->ia_addr)->sin6_addr),
 	    &ip->ip_src, sizeof(ip->ip_src));
+	IFA_UNLOCK(&ia6->ia_ifa);
 	bcopy(in4, &ip->ip_dst, sizeof(ip->ip_dst));
 	ip->ip_p = IPPROTO_IPV6;
 	ip->ip_ttl = ip_stf_ttl;
-	ip->ip_len = m->m_pkthdr.len;	/*host order*/
-	if (ifp->if_flags & IFF_LINK1)
-		ip_ecn_ingress(ECN_ALLOWED, &ip->ip_tos, &tos);
-	else
+	ip->ip_len = m->m_pkthdr.len;   /*host order*/
+	if (ifp->if_flags & IFF_LINK1) {
+		ip_ecn_ingress(ECN_NORMAL, &ip->ip_tos, &tos);
+	} else {
 		ip_ecn_ingress(ECN_NOCARE, &ip->ip_tos, &tos);
+	}
 
-	dst4 = (struct sockaddr_in *)&sc->sc_ro.ro_dst;
-	if (dst4->sin_family != AF_INET ||
+	lck_mtx_lock(&sc->sc_ro_mtx);
+	dst4 = (struct sockaddr_in *)(void *)&sc->sc_ro.ro_dst;
+	if (ROUTE_UNUSABLE(&sc->sc_ro) || dst4->sin_family != AF_INET ||
 	    bcmp(&dst4->sin_addr, &ip->ip_dst, sizeof(ip->ip_dst)) != 0) {
-		/* cache route doesn't match */
-		printf("stf_output: cached route doesn't match \n");
+		ROUTE_RELEASE(&sc->sc_ro);
+		/* cache route doesn't match: always the case during the first use */
 		dst4->sin_family = AF_INET;
 		dst4->sin_len = sizeof(struct sockaddr_in);
 		bcopy(&ip->ip_dst, &dst4->sin_addr, sizeof(dst4->sin_addr));
-		if (sc->sc_ro.ro_rt) {
-			rtfree(sc->sc_ro.ro_rt);
-			sc->sc_ro.ro_rt = NULL;
-		}
 	}
 
-	if (sc->sc_ro.ro_rt == NULL) {
-		rtalloc(&sc->sc_ro);
-		if (sc->sc_ro.ro_rt == NULL) {
-			ifafree(&ia6->ia_ifa);
-			return ENETUNREACH;
-		}
-	}
+	result = ip_output(m, NULL, &sc->sc_ro, IP_OUTARGS, NULL, &ipoa);
+	lck_mtx_unlock(&sc->sc_ro_mtx);
 
-	result = ip_output_list(m, 0, NULL, &sc->sc_ro, 0, NULL, NULL);
 	/* Assumption: ip_output will free mbuf on errors */
 	/* All the output processing is done here, don't let stf_output be called */
-	if (result == 0)
+	if (result == 0) {
 		result = EJUSTRETURN;
+	}
 	*m0 = NULL;
-	ifafree(&ia6->ia_ifa);
+	IFA_REMREF(&ia6->ia_ifa);
 	return result;
 }
 static errno_t
 stf_output(
-	__unused ifnet_t	ifp,
-	__unused mbuf_t	m)
+	__unused ifnet_t        ifp,
+	__unused mbuf_t m)
 {
 	/* All processing is done in stf_pre_output
 	 * this shouldn't be called as the pre_output returns "EJUSTRETURN"
 	 */
 	return 0;
-}	
+}
 
 static int
 stf_checkaddr4(
 	struct stf_softc *sc,
 	const struct in_addr *in,
-	struct ifnet *inifp)	/* incoming interface */
+	struct ifnet *inifp)    /* incoming interface */
 {
 	struct in_ifaddr *ia4;
 
@@ -620,8 +645,9 @@ stf_checkaddr4(
 	 * reject packets with the following address:
 	 * 224.0.0.0/4 0.0.0.0/8 127.0.0.0/8 255.0.0.0/8
 	 */
-	if (IN_MULTICAST(ntohl(in->s_addr)))
+	if (IN_MULTICAST(ntohl(in->s_addr))) {
 		return -1;
+	}
 	switch ((ntohl(in->s_addr) & 0xff000000) >> 24) {
 	case 0: case 127: case 255:
 		return -1;
@@ -630,19 +656,23 @@ stf_checkaddr4(
 	/*
 	 * reject packets with broadcast
 	 */
-	lck_rw_lock_shared(in_ifaddr_rwlock);
+	lck_rw_lock_shared(&in_ifaddr_rwlock);
 	for (ia4 = TAILQ_FIRST(&in_ifaddrhead);
-	     ia4;
-	     ia4 = TAILQ_NEXT(ia4, ia_link))
-	{
-		if ((ia4->ia_ifa.ifa_ifp->if_flags & IFF_BROADCAST) == 0)
+	    ia4;
+	    ia4 = TAILQ_NEXT(ia4, ia_link)) {
+		IFA_LOCK(&ia4->ia_ifa);
+		if ((ia4->ia_ifa.ifa_ifp->if_flags & IFF_BROADCAST) == 0) {
+			IFA_UNLOCK(&ia4->ia_ifa);
 			continue;
+		}
 		if (in->s_addr == ia4->ia_broadaddr.sin_addr.s_addr) {
-			lck_rw_done(in_ifaddr_rwlock);
+			IFA_UNLOCK(&ia4->ia_ifa);
+			lck_rw_done(&in_ifaddr_rwlock);
 			return -1;
 		}
+		IFA_UNLOCK(&ia4->ia_ifa);
 	}
-	lck_rw_done(in_ifaddr_rwlock);
+	lck_rw_done(&in_ifaddr_rwlock);
 
 	/*
 	 * perform ingress filter
@@ -656,8 +686,9 @@ stf_checkaddr4(
 		sin.sin_len = sizeof(struct sockaddr_in);
 		sin.sin_addr = *in;
 		rt = rtalloc1((struct sockaddr *)&sin, 0, 0);
-		if (rt != NULL)
+		if (rt != NULL) {
 			RT_LOCK(rt);
+		}
 		if (rt == NULL || rt->rt_ifp != inifp) {
 #if 1
 			log(LOG_WARNING, "%s: packet from 0x%x dropped "
@@ -681,13 +712,14 @@ static int
 stf_checkaddr6(
 	struct stf_softc *sc,
 	struct in6_addr *in6,
-	struct ifnet *inifp)	/* incoming interface */
+	struct ifnet *inifp)    /* incoming interface */
 {
 	/*
 	 * check 6to4 addresses
 	 */
-	if (IN6_IS_ADDR_6TO4(in6))
+	if (IN6_IS_ADDR_6TO4(in6)) {
 		return stf_checkaddr4(sc, GET_V4(in6), inifp);
+	}
 
 	/*
 	 * reject anything that look suspicious.  the test is implemented
@@ -695,8 +727,9 @@ stf_checkaddr6(
 	 * (1) reject bad packets earlier, and
 	 * (2) to be safe against future ip6_input change.
 	 */
-	if (IN6_IS_ADDR_V4COMPAT(in6) || IN6_IS_ADDR_V4MAPPED(in6))
+	if (IN6_IS_ADDR_V4COMPAT(in6) || IN6_IS_ADDR_V4MAPPED(in6)) {
 		return -1;
+	}
 
 	return 0;
 }
@@ -712,7 +745,7 @@ in_stf_input(
 	u_int8_t otos, itos;
 	int proto;
 	struct ifnet *ifp;
-	struct ifnet_stat_increment_param	stats;
+	struct ifnet_stat_increment_param       stats;
 
 	ip = mtod(m, struct ip *);
 	proto = ip->ip_p;
@@ -733,10 +766,6 @@ in_stf_input(
 
 	ifp = sc->sc_if;
 
-#if MAC_LABEL
-	mac_mbuf_label_associate_ifnet(ifp, m);
-#endif
-
 	/*
 	 * perform sanity check against outer src/dst.
 	 * for source, perform ingress filter as well.
@@ -761,17 +790,18 @@ in_stf_input(
 	}
 
 	itos = (ntohl(ip6.ip6_flow) >> 20) & 0xff;
-	if ((ifnet_flags(ifp) & IFF_LINK1) != 0)
-		ip_ecn_egress(ECN_ALLOWED, &otos, &itos);
-	else
+	if ((ifnet_flags(ifp) & IFF_LINK1) != 0) {
+		ip_ecn_egress(ECN_NORMAL, &otos, &itos);
+	} else {
 		ip_ecn_egress(ECN_NOCARE, &otos, &itos);
+	}
 	ip6.ip6_flow &= ~htonl(0xff << 20);
 	ip6.ip6_flow |= htonl((u_int32_t)itos << 20);
 
 	m->m_pkthdr.rcvif = ifp;
 	mbuf_pkthdr_setheader(m, mbuf_data(m));
 	mbuf_adj(m, off);
-	
+
 	if (ifp->if_bpf) {
 		/* We need to prepend the address family as a four byte field. */
 		u_int32_t af = AF_INET6;
@@ -789,7 +819,7 @@ in_stf_input(
 	stats.bytes_in = mbuf_pkthdr_len(m);
 	mbuf_pkthdr_setrcvif(m, ifp);
 	ifnet_input(ifp, m, &stats);
-	
+
 	return;
 }
 
@@ -807,9 +837,9 @@ stf_rtrequest(
 
 static errno_t
 stf_ioctl(
-	ifnet_t		ifp,
-	u_long		cmd,
-	void		*data)
+	ifnet_t         ifp,
+	u_long          cmd,
+	void            *data)
 {
 	struct ifaddr *ifa;
 	struct ifreq *ifr;
@@ -820,32 +850,45 @@ stf_ioctl(
 	switch (cmd) {
 	case SIOCSIFADDR:
 		ifa = (struct ifaddr *)data;
-		if (ifa == NULL || ifa->ifa_addr->sa_family != AF_INET6) {
+		if (ifa == NULL) {
 			error = EAFNOSUPPORT;
 			break;
 		}
-		sin6 = (struct sockaddr_in6 *)ifa->ifa_addr;
+		IFA_LOCK(ifa);
+		if (ifa->ifa_addr->sa_family != AF_INET6) {
+			IFA_UNLOCK(ifa);
+			error = EAFNOSUPPORT;
+			break;
+		}
+		sin6 = (struct sockaddr_in6 *)(void *)ifa->ifa_addr;
 		if (IN6_IS_ADDR_6TO4(&sin6->sin6_addr)) {
-                        if ( !(ifnet_flags( ifp ) & IFF_UP) ) {
-                                /* do this only if the interface is not already up */
+			if (!(ifnet_flags( ifp ) & IFF_UP)) {
+				/* do this only if the interface is not already up */
 				ifa->ifa_rtrequest = stf_rtrequest;
+				IFA_UNLOCK(ifa);
 				ifnet_set_flags(ifp, IFF_UP, IFF_UP);
+			} else {
+				IFA_UNLOCK(ifa);
 			}
-		} else
+		} else {
+			IFA_UNLOCK(ifa);
 			error = EINVAL;
+		}
+		IFA_LOCK_ASSERT_NOTHELD(ifa);
 		break;
 
 	case SIOCADDMULTI:
 	case SIOCDELMULTI:
 		ifr = (struct ifreq *)data;
-		if (ifr && ifr->ifr_addr.sa_family == AF_INET6)
+		if (ifr && ifr->ifr_addr.sa_family == AF_INET6) {
 			;
-		else
+		} else {
 			error = EAFNOSUPPORT;
+		}
 		break;
 
 	default:
-		error = EINVAL;
+		error = EOPNOTSUPP;
 		break;
 	}
 
