@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1999-2008 Apple Inc.
+ * Copyright (c) 1999-2016 Apple Inc.
  * Copyright (c) 2006-2008 Robert N. M. Watson
  * All rights reserved.
  *
@@ -45,7 +45,6 @@
 #include <sys/vnode_internal.h>
 #include <sys/user.h>
 #include <sys/syscall.h>
-#include <sys/malloc.h>
 #include <sys/un.h>
 #include <sys/sysent.h>
 #include <sys/sysproto.h>
@@ -68,10 +67,8 @@
 
 #include <kern/host.h>
 #include <kern/zalloc.h>
-#include <kern/lock.h>
 #include <kern/sched_prim.h>
 #include <kern/task.h>
-#include <kern/wait_queue.h>
 
 #include <net/route.h>
 
@@ -92,29 +89,29 @@ static thread_t audit_thread;
  * cleared when the next rotation takes place.  It is also protected by
  * audit_worker_sl.
  */
-static int			audit_file_rotate_wait;
-static struct slck 		audit_worker_sl;
-static struct vfs_context	audit_ctx;
-static struct vnode		*audit_vp;
+static int                      audit_file_rotate_wait;
+static struct slck              audit_worker_sl;
+static struct vfs_context       audit_ctx;
+static struct vnode             *audit_vp;
 
-#define	AUDIT_WORKER_SX_INIT()		slck_init(&audit_worker_sl, 	\
-    					    "audit_worker_sl")
-#define	AUDIT_WORKER_SX_XLOCK()		slck_lock(&audit_worker_sl)
-#define	AUDIT_WORKER_SX_XUNLOCK()	slck_unlock(&audit_worker_sl)
-#define	AUDIT_WORKER_SX_ASSERT()	slck_assert(&audit_worker_sl, SL_OWNED)
-#define	AUDIT_WORKER_SX_DESTROY()	slck_destroy(&audit_worker_sl)
+#define AUDIT_WORKER_SX_INIT()          slck_init(&audit_worker_sl,     \
+	                                    "audit_worker_sl")
+#define AUDIT_WORKER_SX_XLOCK()         slck_lock(&audit_worker_sl)
+#define AUDIT_WORKER_SX_XUNLOCK()       slck_unlock(&audit_worker_sl)
+#define AUDIT_WORKER_SX_ASSERT()        slck_assert(&audit_worker_sl, SL_OWNED)
+#define AUDIT_WORKER_SX_DESTROY()       slck_destroy(&audit_worker_sl)
 
 /*
- * The audit_draining flag is set when audit is disabled and the audit
+ * The audit_q_draining flag is set when audit is disabled and the audit
  * worker queue is being drained.
  */
-static int			audit_draining;
+static int                      audit_q_draining;
 
 /*
  * The special kernel audit record, audit_drain_kar, is used to mark the end of
  * the queue when draining it.
  */
-static struct kaudit_record 	audit_drain_kar = {
+static struct kaudit_record     audit_drain_kar = {
 	.k_ar = {
 		.ar_event = AUE_NULL,
 	},
@@ -142,13 +139,15 @@ audit_record_write(struct vnode *vp, struct vfs_context *ctx, void *data,
 	uint64_t temp;
 	off_t file_size;
 
-	AUDIT_WORKER_SX_ASSERT();	/* audit_file_rotate_wait. */
+	AUDIT_WORKER_SX_ASSERT();       /* audit_file_rotate_wait. */
 
-	if (vp == NULL)
+	if (vp == NULL) {
 		return;
+	}
 
-	if (vnode_getwithref(vp))
+	if (vnode_getwithref(vp)) {
 		return /*(ENOENT)*/;
+	}
 
 	mnt_stat = &vp->v_mount->mnt_vfsstat;
 
@@ -158,11 +157,13 @@ audit_record_write(struct vnode *vp, struct vfs_context *ctx, void *data,
 	 * operations to indicate a future inability to write to the file.
 	 */
 	error = vfs_update_vfsstat(vp->v_mount, ctx, VFS_KERNEL_EVENT);
-	if (error)
+	if (error) {
 		goto fail;
+	}
 	error = vnode_size(vp, &file_size, ctx);
-	if (error)
+	if (error) {
 		goto fail;
+	}
 	audit_fstat.af_currsz = (u_quad_t)file_size;
 
 	/*
@@ -203,15 +204,11 @@ audit_record_write(struct vnode *vp, struct vfs_context *ctx, void *data,
 	 */
 	if (audit_qctrl.aq_minfree != 0) {
 		temp = mnt_stat->f_blocks / (100 / audit_qctrl.aq_minfree);
-		if (mnt_stat->f_bfree < temp) {
-			if (ppsratecheck(&last_lowspace_trigger,
-			    &cur_lowspace_trigger, 1)) {
-				(void)audit_send_trigger(
-				    AUDIT_TRIGGER_LOW_SPACE);
-				printf("Warning: audit space low (< %d%% free)"
-				    "on audit log file-system\n",
-				    audit_qctrl.aq_minfree);
-			}
+		if (mnt_stat->f_bfree < temp &&
+		    ppsratecheck(&last_lowspace_trigger,
+		    &cur_lowspace_trigger, 1)) {
+			(void)audit_send_trigger(
+				AUDIT_TRIGGER_LOW_SPACE);
 		}
 	}
 
@@ -243,10 +240,11 @@ audit_record_write(struct vnode *vp, struct vfs_context *ctx, void *data,
 		if ((unsigned long)((audit_q_len + audit_pre_q_len + 1) *
 		    MAX_AUDIT_RECORD_SIZE) / mnt_stat->f_bsize >=
 		    (unsigned long)(mnt_stat->f_bfree)) {
-			if (ppsratecheck(&last_fail, &cur_fail, 1))
+			if (ppsratecheck(&last_fail, &cur_fail, 1)) {
 				printf("audit_record_write: free space "
 				    "below size of audit queue, failing "
 				    "stop\n");
+			}
 			audit_in_failure = 1;
 		} else if (audit_in_failure) {
 			/*
@@ -258,12 +256,13 @@ audit_record_write(struct vnode *vp, struct vfs_context *ctx, void *data,
 	}
 
 	error = vn_rdwr(UIO_WRITE, vp, data, len, (off_t)0, UIO_SYSSPACE,
-	    IO_APPEND|IO_UNIT, vfs_context_ucred(ctx), NULL,
+	    IO_APPEND | IO_UNIT, vfs_context_ucred(ctx), NULL,
 	    vfs_context_proc(ctx));
-	if (error == ENOSPC)
+	if (error == ENOSPC) {
 		goto fail_enospc;
-	else if (error)
+	} else if (error) {
 		goto fail;
+	}
 
 	/*
 	 * Catch completion of a queue drain here; if we're draining and the
@@ -305,9 +304,10 @@ fail:
 	 */
 	if (audit_panic_on_write_fail) {
 		(void)VNOP_FSYNC(vp, MNT_WAIT, ctx);
-		panic("audit_worker: write error %d\n", error);
-	} else if (ppsratecheck(&last_fail, &cur_fail, 1))
+		panic("audit_worker: write error %d", error);
+	} else if (ppsratecheck(&last_fail, &cur_fail, 1)) {
 		printf("audit_worker: write error %d\n", error);
+	}
 	vnode_put(vp);
 }
 
@@ -338,8 +338,9 @@ audit_worker_process_record(struct kaudit_record *ar)
 	    (ar->k_ar_commit & AR_PRESELECT_TRAIL)) {
 		AUDIT_WORKER_SX_XLOCK();
 		trail_locked = 1;
-	} else
+	} else {
 		trail_locked = 0;
+	}
 
 	/*
 	 * First, handle the user record, if any: commit to the system trail
@@ -353,21 +354,25 @@ audit_worker_process_record(struct kaudit_record *ar)
 	}
 
 	if ((ar->k_ar_commit & AR_COMMIT_USER) &&
-	    (ar->k_ar_commit & AR_PRESELECT_USER_PIPE))
+	    (ar->k_ar_commit & AR_PRESELECT_USER_PIPE)) {
 		audit_pipe_submit_user(ar->k_udata, ar->k_ulen);
+	}
 
 	if (!(ar->k_ar_commit & AR_COMMIT_KERNEL) ||
 	    ((ar->k_ar_commit & AR_PRESELECT_PIPE) == 0 &&
-	    (ar->k_ar_commit & AR_PRESELECT_TRAIL) == 0))
+	    (ar->k_ar_commit & AR_PRESELECT_TRAIL) == 0 &&
+	    (ar->k_ar_commit & AR_PRESELECT_FILTER) == 0)) {
 		goto out;
+	}
 
 	auid = ar->k_ar.ar_subj_auid;
 	event = ar->k_ar.ar_event;
 	class = au_event_class(event);
-	if (ar->k_ar.ar_errno == 0)
+	if (ar->k_ar.ar_errno == 0) {
 		sorf = AU_PRS_SUCCESS;
-	else
+	} else {
 		sorf = AU_PRS_FAILURE;
+	}
 
 	error = kaudit_to_bsm(ar, &bsm);
 	switch (error) {
@@ -390,15 +395,26 @@ audit_worker_process_record(struct kaudit_record *ar)
 		audit_record_write(audit_vp, &audit_ctx, bsm->data, bsm->len);
 	}
 
-	if (ar->k_ar_commit & AR_PRESELECT_PIPE)
+	if (ar->k_ar_commit & AR_PRESELECT_PIPE) {
 		audit_pipe_submit(auid, event, class, sorf,
 		    ar->k_ar_commit & AR_PRESELECT_TRAIL, bsm->data,
 		    bsm->len);
+	}
+
+	if (ar->k_ar_commit & AR_PRESELECT_FILTER) {
+		/*
+		 *  XXXss - This needs to be generalized so new filters can
+		 *  be easily plugged in.
+		 */
+		audit_sdev_submit(auid, ar->k_ar.ar_subj_asid, bsm->data,
+		    bsm->len);
+	}
 
 	kau_free(bsm);
 out:
-	if (trail_locked)
+	if (trail_locked) {
 		AUDIT_WORKER_SX_XUNLOCK();
+	}
 }
 
 /*
@@ -410,6 +426,7 @@ out:
  * Note: this means that the effect bound on the size of the pending record
  * queue is 2x the length of the global queue.
  */
+__attribute__((noreturn))
 static void
 audit_worker(void)
 {
@@ -417,7 +434,10 @@ audit_worker(void)
 	struct kaudit_record *ar;
 	int lowater_signal;
 
-	audit_ctx.vc_thread = current_thread();
+	if (audit_ctx.vc_thread == NULL) {
+		audit_ctx.vc_thread = current_thread();
+	}
+
 	TAILQ_INIT(&ar_worklist);
 	mtx_lock(&audit_mtx);
 	while (1) {
@@ -426,8 +446,10 @@ audit_worker(void)
 		/*
 		 * Wait for a record.
 		 */
-		while (TAILQ_EMPTY(&audit_q))
-			cv_wait(&audit_worker_cv, &audit_mtx);
+		while (TAILQ_EMPTY(&audit_q)) {
+			cv_wait_continuation(&audit_worker_cv, &audit_mtx,
+			    (thread_continue_t)audit_worker);
+		}
 
 		/*
 		 * If there are records in the global audit record queue,
@@ -440,18 +462,20 @@ audit_worker(void)
 		while ((ar = TAILQ_FIRST(&audit_q))) {
 			TAILQ_REMOVE(&audit_q, ar, k_q);
 			audit_q_len--;
-			if (audit_q_len == audit_qctrl.aq_lowater)
+			if (audit_q_len == audit_qctrl.aq_lowater) {
 				lowater_signal++;
+			}
 			TAILQ_INSERT_TAIL(&ar_worklist, ar, k_q);
 		}
-		if (lowater_signal)
+		if (lowater_signal) {
 			cv_broadcast(&audit_watermark_cv);
+		}
 
 		mtx_unlock(&audit_mtx);
 		while ((ar = TAILQ_FIRST(&ar_worklist))) {
 			TAILQ_REMOVE(&ar_worklist, ar, k_q);
 			if (ar->k_ar_commit & AR_DRAIN_QUEUE) {
-				audit_draining = 0;
+				audit_q_draining = 0;
 				cv_broadcast(&audit_drain_cv);
 			} else {
 				audit_worker_process_record(ar);
@@ -476,11 +500,43 @@ audit_rotate_vnode(kauth_cred_t cred, struct vnode *vp)
 {
 	kauth_cred_t old_audit_cred;
 	struct vnode *old_audit_vp;
-	int audit_was_enabled;
 
 	KASSERT((cred != NULL && vp != NULL) || (cred == NULL && vp == NULL),
 	    ("audit_rotate_vnode: cred %p vp %p", cred, vp));
 
+
+	mtx_lock(&audit_mtx);
+	if (audit_enabled && (NULL == vp)) {
+		/* Auditing is currently enabled but will be disabled. */
+
+		/*
+		 * Disable auditing now so nothing more is added while the
+		 * audit worker thread is draining the audit record queue.
+		 */
+		audit_enabled = 0;
+
+		/*
+		 * Drain the auditing queue by inserting a drain record at the
+		 * end of the queue and waiting for the audit worker thread
+		 * to find this record and signal that it is done before
+		 * we close the audit trail.
+		 */
+		audit_q_draining = 1;
+		while (audit_q_len >= audit_qctrl.aq_hiwater) {
+			cv_wait(&audit_watermark_cv, &audit_mtx);
+		}
+		TAILQ_INSERT_TAIL(&audit_q, &audit_drain_kar, k_q);
+		audit_q_len++;
+		cv_signal(&audit_worker_cv);
+	}
+
+	/* If the audit queue is draining then wait here until it's done. */
+	while (audit_q_draining) {
+		cv_wait(&audit_drain_cv, &audit_mtx);
+	}
+	mtx_unlock(&audit_mtx);
+
+
 	/*
 	 * Rotate the vnode/cred, and clear the rotate flag so that we will
 	 * send a rotate trigger if the new file fills.
@@ -489,38 +545,11 @@ audit_rotate_vnode(kauth_cred_t cred, struct vnode *vp)
 	old_audit_cred = audit_ctx.vc_ucred;
 	old_audit_vp = audit_vp;
 	audit_ctx.vc_ucred = cred;
+	audit_vp = vp;
 	audit_file_rotate_wait = 0;
-	audit_was_enabled = audit_enabled;
-	if ((audit_enabled = (NULL != vp)))
-		audit_vp = vp;
-	audit_draining = (audit_was_enabled && !audit_enabled);
+	audit_enabled = (audit_vp != NULL);
 	AUDIT_WORKER_SX_XUNLOCK();
 
-	/*
-	 * If audit (was enabled and) is now disabled then drain the audit
-	 * record queue and wait until it is done.
-	 */
-	mtx_lock(&audit_mtx);
-	if (audit_draining) {
-		/*
-		 * Insert the special drain record in the queue.
-		 */
-		while (audit_q_len >= audit_qctrl.aq_hiwater)
-			cv_wait(&audit_watermark_cv, &audit_mtx);
-		TAILQ_INSERT_TAIL(&audit_q, &audit_drain_kar, k_q);
-		audit_q_len++;
-		cv_signal(&audit_worker_cv);
-
-		/*
-		 * Wait for the audit worker thread to signal it is done.
-		 */
-		while (audit_draining)
-			cv_wait(&audit_drain_cv, &audit_mtx);
-
-		audit_vp = NULL;
-	}
-	mtx_unlock(&audit_mtx);
-
 	/*
 	 * If there was an old vnode/credential, close and free.
 	 */
@@ -529,9 +558,10 @@ audit_rotate_vnode(kauth_cred_t cred, struct vnode *vp)
 			vn_close(old_audit_vp, AUDIT_CLOSE_FLAGS,
 			    vfs_context_kernel());
 			vnode_put(old_audit_vp);
-		} else
+		} else {
 			printf("audit_rotate_vnode: Couldn't close "
 			    "audit file.\n");
+		}
 		kauth_cred_unref(&old_audit_cred);
 	}
 }
@@ -539,10 +569,10 @@ audit_rotate_vnode(kauth_cred_t cred, struct vnode *vp)
 void
 audit_worker_init(void)
 {
-
 	AUDIT_WORKER_SX_INIT();
 	kernel_thread_start((thread_continue_t)audit_worker, NULL,
 	    &audit_thread);
-	if (audit_thread == THREAD_NULL)
+	if (audit_thread == THREAD_NULL) {
 		panic("audit_worker_init: Couldn't create audit_worker thread");
+	}
 }
