@@ -1,23 +1,24 @@
 /*
- * $Id: lctx.h,v 1.1.6.2 2006/03/03 23:20:46 msteil Exp $
+ * TODO: remove this file
  */
-
 #ifndef _SYS_LCTX_H_
 #define _SYS_LCTX_H_
 
-#include <sys/syscall.h>
-
 #ifndef KERNEL
+#include <sys/errno.h> /* errno, ENOSYS */
+#include <sys/_types/_pid_t.h> /* pid_t */
 static __inline pid_t
 getlcid(pid_t pid)
 {
-        return (syscall(SYS_getlcid, pid));
+	errno = ENOSYS;
+	return -1;
 }
 
 static __inline int
 setlcid(pid_t pid, pid_t lcid)
 {
-        return (syscall(SYS_setlcid, pid, lcid));
+	errno = ENOSYS;
+	return -1;
 }
 #endif
 
@@ -25,4 +26,4 @@ setlcid(pid_t pid, pid_t lcid)
 #define LCID_REMOVE     (-1)
 #define LCID_CREATE     (0)
 
-#endif	/* !_SYS_LCTX_H_ */
+#endif  /* !_SYS_LCTX_H_ */
