@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2007-2012 Apple Inc. All rights reserved.
+ * Copyright (c) 2000-2006 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_START@
  *
@@ -36,11 +37,30 @@
 #define _OS_OSATOMIC_H
 
 #include <libkern/OSBase.h>
+#include <string.h>
 
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
+#ifdef XNU_KERNEL_PRIVATE
+/*
+ * The macro SAFE_CAST_PTR() casts one type of pointer to another type,
+ * making sure the data the pointer is referencing is the same size.
+ * If it is not, it will cause a "size mismatch" assertion.
+ *
+ * This is to work around "SInt32" being defined as "long" on ILP32 and as "int"
+ * on LP64, which would require an explicit cast to "SInt32*" when for instance
+ * passing an "int*" to OSAddAtomic() - which masks size mismatches.
+ * -- var is used, but sizeof does not evaluate the
+ *    argument, i.e. we're safe against "++" etc. in var --
+ */
+#define __SAFE_CAST_PTR(type, var) \
+	({ _Static_assert(sizeof(*(var)) == sizeof(*(type)NULL), "size mismatch"); ((type)(var)); })
+#else
+#define __SAFE_CAST_PTR(type, var) ((type)(var))
+#endif
+
 /*!
  * @header
  *
@@ -48,8 +68,6 @@ extern "C" {
  * This header declares the OSAtomic group of functions for atomic
  * reading and updating of values.
  */
- 
-#if defined(__i386__) || defined(__x86_64__) || defined(__arm__)
 
 /*!
  * @function OSCompareAndSwap64
@@ -61,13 +79,11 @@ extern "C" {
  * See OSCompareAndSwap.
  */
 extern Boolean OSCompareAndSwap64(
-    UInt64            oldValue,
-    UInt64            newValue,
-    volatile UInt64 * address);
-
-#endif /* defined(__i386__) || defined(__x86_64__) */
-
-#if defined(__i386__) || defined(__x86_64__) || defined(__arm__)
+	UInt64            oldValue,
+	UInt64            newValue,
+	volatile UInt64 * address);
+#define OSCompareAndSwap64(a, b, c) \
+	(OSCompareAndSwap64(a, b, __SAFE_CAST_PTR(volatile UInt64*,c)))
 
 /*!
  * @function OSAddAtomic64
@@ -79,8 +95,10 @@ extern Boolean OSCompareAndSwap64(
  * See OSAddAtomic.
  */
 extern SInt64 OSAddAtomic64(
-    SInt64            theAmount,
-    volatile SInt64 * address);
+	SInt64            theAmount,
+	volatile SInt64 * address);
+#define OSAddAtomic64(a, b) \
+	(OSAddAtomic64(a, __SAFE_CAST_PTR(volatile SInt64*,b)))
 
 /*!
  * @function OSIncrementAtomic64
@@ -90,10 +108,11 @@ extern SInt64 OSAddAtomic64(
  *
  * @discussion
  * See OSIncrementAtomic.
-*/
-inline static SInt64 OSIncrementAtomic64(volatile SInt64 * address)
+ */
+inline static SInt64
+OSIncrementAtomic64(volatile SInt64 * address)
 {
-    return OSAddAtomic64(1LL, address);
+	return OSAddAtomic64(1LL, address);
 }
 
 /*!
@@ -104,14 +123,13 @@ inline static SInt64 OSIncrementAtomic64(volatile SInt64 * address)
  *
  * @discussion
  * See OSDecrementAtomic.
-*/
-inline static SInt64 OSDecrementAtomic64(volatile SInt64 * address)
+ */
+inline static SInt64
+OSDecrementAtomic64(volatile SInt64 * address)
 {
-    return OSAddAtomic64(-1LL, address);
+	return OSAddAtomic64(-1LL, address);
 }
 
-#endif  /* defined(__i386__) || defined(__x86_64__) || defined(__arm__) */
-
 #if XNU_KERNEL_PRIVATE
 /* Not to be included in headerdoc.
  *
@@ -124,8 +142,10 @@ inline static SInt64 OSDecrementAtomic64(volatile SInt64 * address)
  * See OSAddAtomic.
  */
 extern long OSAddAtomicLong(
-    long            theAmount,
-    volatile long * address);
+	long            theAmount,
+	volatile long * address);
+#define OSAddAtomicLong(a, b) \
+	(OSAddAtomicLong(a, __SAFE_CAST_PTR(volatile long*,b)))
 
 /* Not to be included in headerdoc.
  *
@@ -136,10 +156,11 @@ extern long OSAddAtomicLong(
  *
  * @discussion
  * See OSIncrementAtomic.
-*/
-inline static long OSIncrementAtomicLong(volatile long * address)
+ */
+inline static long
+OSIncrementAtomicLong(volatile long * address)
 {
-    return OSAddAtomicLong(1L, address);
+	return OSAddAtomicLong(1L, address);
 }
 
 /* Not to be included in headerdoc.
@@ -150,27 +171,61 @@ inline static long OSIncrementAtomicLong(volatile long * address)
  * 32/64-bit decrement, depending on sizeof(long)
  *@discussion See OSDecrementAtomic.
  */
-inline static long OSDecrementAtomicLong(volatile long * address)
+inline static long
+OSDecrementAtomicLong(volatile long * address)
 {
-    return OSAddAtomicLong(-1L, address);
+	return OSAddAtomicLong(-1L, address);
 }
 #endif /* XNU_KERNEL_PRIVATE */
 
-/*
- * The macro SAFE_CAST_PTR() casts one type of pointer to another type, making sure
- * the data the pointer is referencing is the same size. If it is not, it will cause
- * a division by zero compiler warning. This is to work around "SInt32" being defined
- * as "long" on ILP32 and as "int" on LP64, which would require an explicit cast to
- * "SInt32*" when for instance passing an "int*" to OSAddAtomic() - which masks size
- * mismatches.
- * -- var is used twice, but sizeof does not evaluate the
- *    argument, i.e. we're safe against "++" etc. in var --
+#if XNU_KERNEL_PRIVATE
+/*!
+ * @function OSCompareAndSwap8
+ *
+ * @abstract
+ * Compare and swap operation, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
+ *
+ * @discussion
+ * The OSCompareAndSwap8 function compares the value at the specified address with oldVal. The value of newValue is written to the address only if oldValue and the value at the address are equal. OSCompareAndSwap returns true if newValue is written to the address; otherwise, it returns false.
+ *
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ *
+ * @param oldValue The value to compare at address.
+ * @param newValue The value to write to address if oldValue compares true.
+ * @param address The byte aligned address of the data to update atomically.
+ * @result true if newValue was written to the address.
  */
-#ifdef XNU_KERNEL_PRIVATE
-#define SAFE_CAST_PTR(type, var) (((type)(var))+(0/(sizeof(*var) == sizeof(*(type)0) ? 1 : 0)))
-#else
-#define SAFE_CAST_PTR(type, var) ((type)(var))
-#endif
+extern Boolean OSCompareAndSwap8(
+	UInt8            oldValue,
+	UInt8            newValue,
+	volatile UInt8 * address);
+#define OSCompareAndSwap8(a, b, c) \
+	(OSCompareAndSwap8(a, b, __SAFE_CAST_PTR(volatile UInt8*,c)))
+
+/*!
+ * @function OSCompareAndSwap16
+ *
+ * @abstract
+ * Compare and swap operation, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
+ *
+ * @discussion
+ * The OSCompareAndSwap16 function compares the value at the specified address with oldVal. The value of newValue is written to the address only if oldValue and the value at the address are equal. OSCompareAndSwap returns true if newValue is written to the address; otherwise, it returns false.
+ *
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ *
+ * @param oldValue The value to compare at address.
+ * @param newValue The value to write to address if oldValue compares true.
+ * @param address The 2-byte aligned address of the data to update atomically.
+ * @result true if newValue was written to the address.
+ */
+extern Boolean OSCompareAndSwap16(
+	UInt16            oldValue,
+	UInt16            newValue,
+	volatile UInt16 * address);
+#define OSCompareAndSwap16(a, b, c) \
+	(OSCompareAndSwap16(a, b, __SAFE_CAST_PTR(volatile UInt16*,c)))
+
+#endif /* XNU_KERNEL_PRIVATE */
 
 /*!
  * @function OSCompareAndSwap
@@ -189,11 +244,11 @@ inline static long OSDecrementAtomicLong(volatile long * address)
  * @result true if newValue was written to the address.
  */
 extern Boolean OSCompareAndSwap(
-    UInt32            oldValue,
-    UInt32            newValue,
-    volatile UInt32 * address);
+	UInt32            oldValue,
+	UInt32            newValue,
+	volatile UInt32 * address);
 #define OSCompareAndSwap(a, b, c) \
-	(OSCompareAndSwap(a, b, SAFE_CAST_PTR(volatile UInt32*,c)))
+	(OSCompareAndSwap(a, b, __SAFE_CAST_PTR(volatile UInt32*,c)))
 
 /*!
  * @function OSCompareAndSwapPtr
@@ -211,11 +266,11 @@ extern Boolean OSCompareAndSwap(
  * @result true if newValue was written to the address.
  */
 extern Boolean OSCompareAndSwapPtr(
-    void            * oldValue,
-    void            * newValue,
-    void * volatile * address);
+	void            * oldValue,
+	void            * newValue,
+	void * volatile * address);
 #define OSCompareAndSwapPtr(a, b, c) \
-	(OSCompareAndSwapPtr(a, b, SAFE_CAST_PTR(void * volatile *,c)))
+	(OSCompareAndSwapPtr(a, b, __SAFE_CAST_PTR(void * volatile *,c)))
 
 /*!
  * @function OSAddAtomic
@@ -226,16 +281,16 @@ extern Boolean OSCompareAndSwapPtr(
  * @discussion
  * The OSAddAtomic function adds the specified amount to the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param amount The amount to add.
  * @param address The 4-byte aligned address of the value to update atomically.
  * @result The value before the addition
  */
 extern SInt32 OSAddAtomic(
-    SInt32            amount,
-    volatile SInt32 * address);
+	SInt32            amount,
+	volatile SInt32 * address);
 #define OSAddAtomic(a, b) \
-	(OSAddAtomic(a, SAFE_CAST_PTR(volatile SInt32*,b)))
+	(OSAddAtomic(a, __SAFE_CAST_PTR(volatile SInt32*,b)))
 
 /*!
  * @function OSAddAtomic16
@@ -246,14 +301,13 @@ extern SInt32 OSAddAtomic(
  * @discussion
  * The OSAddAtomic16 function adds the specified amount to the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
- * @param amount The amount to add.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param address The 2-byte aligned address of the value to update atomically.
  * @result The value before the addition
  */
 extern SInt16 OSAddAtomic16(
-    SInt32            amount,
-    volatile SInt16 * address);
+	SInt32            amount,
+	volatile SInt16 * address);
 
 /*!
  * @function OSAddAtomic8
@@ -264,14 +318,14 @@ extern SInt16 OSAddAtomic16(
  * @discussion
  * The OSAddAtomic8 function adds the specified amount to the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param amount The amount to add.
  * @param address The address of the value to update atomically.
  * @result The value before the addition.
  */
 extern SInt8 OSAddAtomic8(
-    SInt32           amount,
-    volatile SInt8 * address);
+	SInt32           amount,
+	volatile SInt8 * address);
 
 /*!
  * @function OSIncrementAtomic
@@ -288,7 +342,7 @@ extern SInt8 OSAddAtomic8(
  */
 extern SInt32 OSIncrementAtomic(volatile SInt32 * address);
 #define OSIncrementAtomic(a) \
-	(OSIncrementAtomic(SAFE_CAST_PTR(volatile SInt32*,a)))
+	(OSIncrementAtomic(__SAFE_CAST_PTR(volatile SInt32*,a)))
 
 /*!
  * @function OSIncrementAtomic16
@@ -299,7 +353,7 @@ extern SInt32 OSIncrementAtomic(volatile SInt32 * address);
  * @discussion
  * The OSIncrementAtomic16 function increments the value at the specified address by one and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param address The 2-byte aligned address of the value to update atomically.
  * @result The value before the increment.
  */
@@ -314,7 +368,7 @@ extern SInt16 OSIncrementAtomic16(volatile SInt16 * address);
  * @discussion
  * The OSIncrementAtomic8 function increments the value at the specified address by one and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param address The address of the value to update atomically.
  * @result The value before the increment.
  */
@@ -329,13 +383,13 @@ extern SInt8 OSIncrementAtomic8(volatile SInt8 * address);
  * @discussion
  * The OSDecrementAtomic function decrements the value at the specified address by one and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param address The 4-byte aligned address of the value to update atomically.
  * @result The value before the decrement.
  */
 extern SInt32 OSDecrementAtomic(volatile SInt32 * address);
 #define OSDecrementAtomic(a) \
-	(OSDecrementAtomic(SAFE_CAST_PTR(volatile SInt32*,a)))
+	(OSDecrementAtomic(__SAFE_CAST_PTR(volatile SInt32*,a)))
 
 /*!
  * @function OSDecrementAtomic16
@@ -346,7 +400,7 @@ extern SInt32 OSDecrementAtomic(volatile SInt32 * address);
  * @discussion
  * The OSDecrementAtomic16 function decrements the value at the specified address by one and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param address The 2-byte aligned address of the value to update atomically.
  * @result The value before the decrement.
  */
@@ -361,7 +415,7 @@ extern SInt16 OSDecrementAtomic16(volatile SInt16 * address);
  * @discussion
  * The OSDecrementAtomic8 function decrements the value at the specified address by one and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers.
  * @param address The address of the value to update atomically.
  * @result The value before the decrement.
  */
@@ -376,16 +430,16 @@ extern SInt8 OSDecrementAtomic8(volatile SInt8 * address);
  * @discussion
  * The OSBitAndAtomic function logically ands the bits of the specified mask into the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Previous incarnations of this function incorporated a memory barrier on systems with weakly-ordered memory architectures, but current versions contain no barriers..
  * @param mask The mask to logically and with the value.
  * @param address The 4-byte aligned address of the value to update atomically.
  * @result The value before the bitwise operation
  */
 extern UInt32 OSBitAndAtomic(
-    UInt32            mask,
-    volatile UInt32 * address);
+	UInt32            mask,
+	volatile UInt32 * address);
 #define OSBitAndAtomic(a, b) \
-	(OSBitAndAtomic(a, SAFE_CAST_PTR(volatile UInt32*,b)))
+	(OSBitAndAtomic(a, __SAFE_CAST_PTR(volatile UInt32*,b)))
 
 /*!
  * @function OSBitAndAtomic16
@@ -396,14 +450,14 @@ extern UInt32 OSBitAndAtomic(
  * @discussion
  * The OSBitAndAtomic16 function logically ands the bits of the specified mask into the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  * @param mask The mask to logically and with the value.
  * @param address The 2-byte aligned address of the value to update atomically.
  * @result The value before the bitwise operation.
  */
 extern UInt16 OSBitAndAtomic16(
-    UInt32            mask,
-    volatile UInt16 * address);
+	UInt32            mask,
+	volatile UInt16 * address);
 
 /*!
  * @function OSBitAndAtomic8
@@ -414,14 +468,14 @@ extern UInt16 OSBitAndAtomic16(
  * @discussion
  * The OSBitAndAtomic8 function logically ands the bits of the specified mask into the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  * @param mask The mask to logically and with the value.
  * @param address The address of the value to update atomically.
  * @result The value before the bitwise operation.
  */
 extern UInt8 OSBitAndAtomic8(
-    UInt32           mask,
-    volatile UInt8 * address);
+	UInt32           mask,
+	volatile UInt8 * address);
 
 /*!
  * @function OSBitOrAtomic
@@ -432,16 +486,16 @@ extern UInt8 OSBitAndAtomic8(
  * @discussion
  * The OSBitOrAtomic function logically ors the bits of the specified mask into the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  * @param mask The mask to logically or with the value.
  * @param address The 4-byte aligned address of the value to update atomically.
  * @result The value before the bitwise operation.
  */
 extern UInt32 OSBitOrAtomic(
-    UInt32            mask,
-    volatile UInt32 * address);
+	UInt32            mask,
+	volatile UInt32 * address);
 #define OSBitOrAtomic(a, b) \
-	(OSBitOrAtomic(a, SAFE_CAST_PTR(volatile UInt32*,b)))
+	(OSBitOrAtomic(a, __SAFE_CAST_PTR(volatile UInt32*,b)))
 
 /*!
  * @function OSBitOrAtomic16
@@ -452,14 +506,14 @@ extern UInt32 OSBitOrAtomic(
  * @discussion
  * The OSBitOrAtomic16 function logically ors the bits of the specified mask into the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  * @param mask The mask to logically or with the value.
  * @param address The 2-byte aligned address of the value to update atomically.
  * @result The value before the bitwise operation.
  */
 extern UInt16 OSBitOrAtomic16(
-    UInt32            mask,
-    volatile UInt16 * address);
+	UInt32            mask,
+	volatile UInt16 * address);
 
 /*!
  * @function OSBitOrAtomic8
@@ -467,7 +521,7 @@ extern UInt16 OSBitOrAtomic16(
  * @abstract
  * 8-bit logical or operation, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  *
  * @discussion
  * The OSBitOrAtomic8 function logically ors the bits of the specified mask into the value at the specified address and returns the original value.
@@ -476,8 +530,8 @@ extern UInt16 OSBitOrAtomic16(
  * @result The value before the bitwise operation.
  */
 extern UInt8 OSBitOrAtomic8(
-    UInt32           mask,
-    volatile UInt8 * address);
+	UInt32           mask,
+	volatile UInt8 * address);
 
 /*!
  * @function OSBitXorAtomic
@@ -485,7 +539,7 @@ extern UInt8 OSBitOrAtomic8(
  * @abstract
  * 32-bit logical xor operation, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  *
  * @discussion
  * The OSBitXorAtomic function logically xors the bits of the specified mask into the value at the specified address and returns the original value.
@@ -494,10 +548,10 @@ extern UInt8 OSBitOrAtomic8(
  * @result The value before the bitwise operation.
  */
 extern UInt32 OSBitXorAtomic(
-    UInt32            mask,
-    volatile UInt32 * address);
+	UInt32            mask,
+	volatile UInt32 * address);
 #define OSBitXorAtomic(a, b) \
-	(OSBitXorAtomic(a, SAFE_CAST_PTR(volatile UInt32*,b)))
+	(OSBitXorAtomic(a, __SAFE_CAST_PTR(volatile UInt32*,b)))
 
 /*!
  * @function OSBitXorAtomic16
@@ -508,14 +562,14 @@ extern UInt32 OSBitXorAtomic(
  * @discussion
  * The OSBitXorAtomic16 function logically xors the bits of the specified mask into the value at the specified address and returns the original value.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  * @param mask The mask to logically or with the value.
  * @param address The 2-byte aligned address of the value to update atomically.
  * @result The value before the bitwise operation.
  */
 extern UInt16 OSBitXorAtomic16(
-    UInt32            mask,
-    volatile UInt16 * address);
+	UInt32            mask,
+	volatile UInt16 * address);
 
 /*!
  * @function OSBitXorAtomic8
@@ -523,7 +577,7 @@ extern UInt16 OSBitXorAtomic16(
  * @abstract
  * 8-bit logical xor operation, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  *
  * @discussion
  * The OSBitXorAtomic8 function logically xors the bits of the specified mask into the value at the specified address and returns the original value.
@@ -532,8 +586,8 @@ extern UInt16 OSBitXorAtomic16(
  * @result The value before the bitwise operation.
  */
 extern UInt8 OSBitXorAtomic8(
-    UInt32           mask,
-    volatile UInt8 * address);
+	UInt32           mask,
+	volatile UInt8 * address);
 
 /*!
  * @function OSTestAndSet
@@ -541,17 +595,17 @@ extern UInt8 OSBitXorAtomic8(
  * @abstract
  * Bit test and set operation, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
  *
  * @discussion
  * The OSTestAndSet function sets a single bit in a byte at a specified address. It returns true if the bit was already set, false otherwise.
- * @param bit The bit number in the range 0 through 7.
+ * @param bit The bit number in the range 0 through 7. Bit 0 is the most significant.
  * @param startAddress The address of the byte to update atomically.
  * @result true if the bit was already set, false otherwise.
  */
 extern Boolean OSTestAndSet(
-    UInt32           bit,
-    volatile UInt8 * startAddress);
+	UInt32           bit,
+	volatile UInt8 * startAddress);
 
 /*!
  * @function OSTestAndClear
@@ -562,53 +616,63 @@ extern Boolean OSTestAndSet(
  * @discussion
  * The OSTestAndClear function clears a single bit in a byte at a specified address. It returns true if the bit was already clear, false otherwise.
  *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
- * @param bit The bit number in the range 0 through 7.
+ * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device.
+ * @param bit The bit number in the range 0 through 7. Bit 0 is the most significant.
  * @param startAddress The address of the byte to update atomically.
  * @result true if the bit was already clear, false otherwise.
  */
 extern Boolean OSTestAndClear(
-    UInt32           bit,
-    volatile UInt8 * startAddress);
-
-#ifdef __ppc__
-/*!
- * @function OSEnqueueAtomic
- *
- * @abstract
- * Singly linked list head insertion, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
- *
- * @discussion
- * The OSEnqueueAtomic function places an element at the head of a single linked list, which is specified with the address of a head pointer, listHead. The element structure has a next field whose offset is specified.
- *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
- * @param listHead The address of a head pointer for the list .
- * @param element The list element to insert at the head of the list.
- * @param elementNextFieldOffset The byte offset into the element where a pointer to the next element in the list is stored.
- */
-extern void OSEnqueueAtomic(
-    void * volatile * listHead,
-    void            * element,
-    SInt32            elementNextFieldOffset);
+	UInt32           bit,
+	volatile UInt8 * startAddress);
 
 /*!
- * @function OSDequeueAtomic
+ * @defined OS_SPINLOCK_INIT
  *
  * @abstract
- * Singly linked list element head removal, performed atomically with respect to all devices that participate in the coherency architecture of the platform.
+ * The default value for an OSSpinLock.
  *
  * @discussion
- * The OSDequeueAtomic function removes an element from the head of a single linked list, which is specified with the address of a head pointer, listHead. The element structure has a next field whose offset is specified.
- *
- * This function guarantees atomicity only with main system memory. It is specifically unsuitable for use on noncacheable memory such as that in devices; this function cannot guarantee atomicity, for example, on memory mapped from a PCI device. Additionally, this function incorporates a memory barrier on systems with weakly-ordered memory architectures.
- * @param listHead The address of a head pointer for the list .
- * @param elementNextFieldOffset The byte offset into the element where a pointer to the next element in the list is stored.
- * @result A removed element, or zero if the list is empty.
+ * The convention is that unlocked is zero, locked is nonzero.
  */
-extern void * OSDequeueAtomic(
-    void * volatile * listHead,
-    SInt32            elementNextFieldOffset);
-#endif /* __ppc__ */
+#define OS_SPINLOCK_INIT 0
+
+/*!
+ * @typedef OSSpinLock
+ *
+ * @abstract
+ * Data type for a spinlock.
+ *
+ * @discussion
+ * You should always initialize a spinlock to OS_SPINLOCK_INIT before using it.
+ */
+typedef SInt32 OSSpinLock;
+
+#ifdef PRIVATE
+/*!
+ * @function OSSpinLockTry
+ *
+ * @abstract
+ * Locks a spinlock if it would not block.
+ *
+ * @discussion
+ * Multiprocessor locks used within the shared memory area between the kernel and event system.  These must work in both user and kernel mode.
+ *
+ * @result
+ * Returns false if the lock was already held by another thread, true if it took the lock successfully.
+ */
+extern Boolean OSSpinLockTry(volatile OSSpinLock * lock);
+
+/*!
+ * @function OSSpinLockUnlock
+ *
+ * @abstract
+ * Unlocks a spinlock.
+ *
+ * @discussion
+ * Unlocks a spinlock.
+ */
+extern void OSSpinLockUnlock(volatile OSSpinLock * lock);
+#endif /* PRIVATE */
 
 /*!
  * @function OSSynchronizeIO
@@ -619,12 +683,34 @@ extern void * OSDequeueAtomic(
  * @discussion
  * The OSSynchronizeIO routine ensures orderly load and store operations to noncached memory mapped I/O devices. It executes the eieio instruction on PowerPC processors.
  */
-static __inline__ void OSSynchronizeIO(void)
+#if defined(__arm__) || defined(__arm64__)
+extern void OSSynchronizeIO(void);
+#else
+static __inline__ void
+OSSynchronizeIO(void)
 {
-#if defined(__ppc__)
-        __asm__ ("eieio");
-#endif
 }
+#endif
+
+#if     defined(KERNEL_PRIVATE)
+
+#if     defined(__arm__) || defined(__arm64__)
+static inline void
+OSMemoryBarrier(void)
+{
+	__asm__ volatile ("dmb ish" ::: "memory");
+}
+#elif defined(__i386__) || defined(__x86_64__)
+#if     defined(XNU_KERNEL_PRIVATE)
+static inline void
+OSMemoryBarrier(void)
+{
+	__asm__ volatile ("mfence" ::: "memory");
+}
+#endif /* XNU_KERNEL_PRIVATE */
+#endif
+
+#endif /* KERNEL_PRIVATE */
 
 #if defined(__cplusplus)
 }
