@@ -30,20 +30,20 @@
 #define _LIBKERN_OSKEXTLIB_H
 
 #include <sys/cdefs.h>
-__BEGIN_DECLS
-
 #include <stdint.h>
 #include <mach/kmod.h>
 #include <mach/vm_types.h>
+#include <uuid/uuid.h>
 
 #ifdef KERNEL
 #include <libkern/OSTypes.h>
 #include <libkern/OSReturn.h>
 #else
-#include <CoreFoundation/CoreFoundation.h>
-#include <System/libkern/OSReturn.h>
+#include <libkern/OSReturn.h>
 #endif /* KERNEL */
 
+__BEGIN_DECLS
+
 /*!
  * @header
  *
@@ -237,6 +237,44 @@ __BEGIN_DECLS
  */
 #define kOSKextReturnStopping                        libkern_kext_err(0x1a)
 
+/*!
+ * @define   kOSKextReturnSystemPolicy
+ * @abstract The kext was prevented from loading due to system policy.
+ */
+#define kOSKextReturnSystemPolicy                    libkern_kext_err(0x1b)
+
+/*!
+ * @define   kOSKextReturnKCLoadFailure
+ * @abstract Loading of the System KC failed
+ */
+#define kOSKextReturnKCLoadFailure                  libkern_kext_err(0x1c)
+
+/*!
+ * @define   kOSKextReturnKCLoadFailureSystemKC
+ * @abstract Loading of the System KC failed
+ *
+ * This a sub-code of kOSKextReturnKCLoadFailure. It can be OR'd together
+ * with: kOSKextReturnKCLoadFailureAuxKC
+ *
+ * If both the System and Aux KCs fail to load, then the error code will be:
+ * libkern_kext_err(0x1f)
+ */
+#define kOSKextReturnKCLoadFailureSystemKC          libkern_kext_err(0x1d)
+
+/*!
+ * @define   kOSKextReturnKCLoadFailureAuxKC
+ * @abstract Loading of the Aux KC failed
+ *
+ * This a sub-code of kOSKextReturnKCLoadFailure. It can be OR'd together
+ * with: kOSKextReturnKCLoadFailureSystemKC
+ *
+ * If both the System and Aux KCs fail to load, then the error code will be:
+ * libkern_kext_err(0x1f)
+ */
+#define kOSKextReturnKCLoadFailureAuxKC             libkern_kext_err(0x1e)
+
+/* next available error is: libkern_kext_err(0x20) */
+
 #if PRAGMA_MARK
 #pragma mark -
 /********************************************************************/
@@ -254,9 +292,13 @@ __BEGIN_DECLS
 /* Define C-string versions of the CFBundle keys for use in the kernel.
  */
 #define kCFBundleIdentifierKey                  "CFBundleIdentifier"
+#define kCFBundleIdentifierKernelKey            "CFBundleIdentifierKernel"
 #define kCFBundleVersionKey                     "CFBundleVersion"
 #define kCFBundleNameKey                        "CFBundleName"
 #define kCFBundleExecutableKey                  "CFBundleExecutable"
+#define kCFBundlePackageTypeKey                 "CFBundlePackageType"
+#define kCFBundleDriverKitUUIDKey               "CFBundleDriverKitUUID"
+#define kCFBundleDriverKitExecutableKey         "CFBundleUEXTExecutable"
 #endif /* KERNEL */
 
 /*!
@@ -312,6 +354,13 @@ __BEGIN_DECLS
  */
 #define kOSBundleRequiredKey                    "OSBundleRequired"
 
+/*!
+ * @define   kOSBundleRequireExplicitLoadKey
+ * @abstract A boolean value indicating whether the kext requires an
+ *           explicit kextload in order to start/match.
+ */
+#define kOSBundleRequireExplicitLoadKey         "OSBundleRequireExplicitLoad"
+
 /*!
  * @define   kOSBundleAllowUserLoadKey
  * @abstract A boolean value indicating whether
@@ -325,6 +374,15 @@ __BEGIN_DECLS
  */
 #define kOSBundleAllowUserLoadKey               "OSBundleAllowUserLoad"
 
+/*!
+ * @define   kOSBundleAllowUserTerminateKey
+ * @abstract A boolean value indicating whether the kextunload tool
+ *           is allowed to issue IOService terminate to classes defined in this kext.
+ * @discussion A boolean value indicating whether the kextunload tool
+ *           is allowed to issue IOService terminate to classes defined in this kext.
+ */
+#define kOSBundleAllowUserTerminateKey          "OSBundleAllowUserTerminate"
+
 /*!
  * @define   kOSKernelResourceKey
  * @abstract A boolean value indicating whether the kext represents a built-in
@@ -332,6 +390,13 @@ __BEGIN_DECLS
  */
 #define kOSKernelResourceKey                    "OSKernelResource"
 
+/*!
+ * @define   kOSKextVariantOverrideKey
+ * @abstract A dictionary with target names as key and a target-specific variant
+ *           name as value.
+ */
+#define kOSKextVariantOverrideKey               "OSKextVariantOverride"
+
 /*!
  * @define   kIOKitPersonalitiesKey
  * @abstract A dictionary of dictionaries used in matching for I/O Kit drivers.
@@ -346,6 +411,23 @@ __BEGIN_DECLS
  */
 #define kIOPersonalityPublisherKey              "IOPersonalityPublisher"
 
+#if CONFIG_KEC_FIPS
+/*
+ * @define   kAppleTextHashesKey
+ * @abstract A dictionary conataining hashes for corecrypto kext.
+ */
+#define kAppleTextHashesKey                     "AppleTextHashes"
+#endif
+
+/*!
+ * @define   kOSMutableSegmentCopy
+ * @abstract A boolean value indicating whether the kext requires a copy of
+ *           its mutable segments to be kept in memory, and then reset when the kext
+ *           unloads. This should be used with caution as it will increase the
+ *           amount of memory used by the kext.
+ */
+#define kOSMutableSegmentCopy                   "OSMutableSegmentCopy"
+
 
 #if PRAGMA_MARK
 /********************************************************************/
@@ -386,61 +468,89 @@ __BEGIN_DECLS
  */
 
 /*!
-* @define   kOSKextKernelIdentifier
-* @abstract
-* This is the CFBundleIdentifier user for the kernel itself.
-*/
+ * @define   kOSKextKernelIdentifier
+ * @abstract
+ * This is the CFBundleIdentifier user for the kernel itself.
+ */
 #define kOSKextKernelIdentifier                 "__kernel__"
 
+
 /*!
-* @define   kOSBundleRequiredRoot
-* @abstract
-* This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
-* value indicates that the kext may be needed to mount the root filesystem
-* whether starting from a local or a network volume.
-*/
+ * @define  kOSKextBundlePackageTypeKext
+ * @abstract
+ * The bundle type value for Kernel Extensions.
+ */
+#define kOSKextBundlePackageTypeKext        "KEXT"
+
+/*!
+ * @define  kOSKextBundlePackageTypeDriverKit
+ * @abstract
+ * The bundle type value for Driver Extensions.
+ */
+#define kOSKextBundlePackageTypeDriverKit   "DEXT"
+
+/*!
+ * @define   kOSBundleRequiredRoot
+ * @abstract
+ * This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
+ * value indicates that the kext may be needed to mount the root filesystem
+ * whether starting from a local or a network volume.
+ */
 #define kOSBundleRequiredRoot                   "Root"
 
 /*!
-* @define   kOSBundleRequiredLocalRoot
-* @abstract
-* This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
-* value indicates that the kext may be needed to mount the root filesystem
-* when starting from a local disk.
-*/
+ * @define   kOSBundleRequiredLocalRoot
+ * @abstract
+ * This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
+ * value indicates that the kext may be needed to mount the root filesystem
+ * when starting from a local disk.
+ */
 #define kOSBundleRequiredLocalRoot              "Local-Root"
 
 /*!
-* @define   kOSBundleRequiredNetworkRoot
-* @abstract
-* This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
-* value indicates that the kext may be needed to mount the root filesystem
-* when starting over a network connection.
-*/
+ * @define   kOSBundleRequiredNetworkRoot
+ * @abstract
+ * This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
+ * value indicates that the kext may be needed to mount the root filesystem
+ * when starting over a network connection.
+ */
 #define kOSBundleRequiredNetworkRoot            "Network-Root"
 
 /*!
-* @define   kOSBundleRequiredSafeBoot
-* @abstract
-* This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
-* value indicates that the kext can be loaded during a safe startup.
-* This value does not normally cause the kext to be read by the booter
-* or included in startup kext caches.
-*/
+ * @define   kOSBundleRequiredSafeBoot
+ * @abstract
+ * This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
+ * value indicates that the kext can be loaded during a safe startup.
+ * This value does not normally cause the kext to be read by the booter
+ * or included in startup kext caches.
+ */
 #define kOSBundleRequiredSafeBoot               "Safe Boot"
 
 /*!
-* @define   kOSBundleRequiredConsole
-* @abstract
-* This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
-* value indicates that the kext may be needed for console access
-* (specifically in a single-user startup when
-* <code>@link //apple_ref/doc/man/8/kextd kextd(8)@/link</code>.
-* does not run)
-* and should be loaded during early startup.
-*/
+ * @define   kOSBundleRequiredConsole
+ * @abstract
+ * This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
+ * value indicates that the kext may be needed for console access
+ * (specifically in a single-user startup when
+ * <code>@link //apple_ref/doc/man/8/kextd kextd(8)@/link</code>.
+ * does not run)
+ * and should be loaded during early startup.
+ */
 #define kOSBundleRequiredConsole                "Console"
 
+/*!
+ * @define   kOSBundleRequiredDriverKit
+ * @abstract
+ * This <code>@link kOSBundleRequiredKey OSBundleRequired@/link</code>
+ * value indicates that the driver extension's (DriverKit driver's)
+ * personalities must be present in the kernel at early boot (specifically
+ * before <code>@link //apple_ref/doc/man/8/kextd kextd(8)@/link</code> starts)
+ * in order to compete with kexts built into the prelinkedkernel. Note that
+ * kextd is still required to launch the user space driver binary. The IOKit
+ * matching will happen during early boot, and the actual driver launch
+ * will happen after kextd starts.
+ */
+#define kOSBundleRequiredDriverKit              "DriverKit"
 
 #if PRAGMA_MARK
 #pragma mark -
@@ -454,6 +564,7 @@ __BEGIN_DECLS
  * about itself.
  */
 
+#ifdef KERNEL
 /*!
  * @typedef OSKextLoadTag
  *
@@ -473,6 +584,22 @@ __BEGIN_DECLS
  * <code>@link OSKextReleaseKextWithLoadTag
  * OSKextReleaseKextWithLoadTag@/link</code>.
  */
+#else
+/*!
+ * @typedef OSKextLoadTag
+ *
+ * @abstract
+ * A unique identifier assigned to a loaded instanace of a kext.
+ *
+ * @discussion
+ * If a kext is unloaded and later reloaded, the new instance
+ * has a different load tag.
+ *
+ * A kext can get its own load tag in the <code>kmod_info_t</code>
+ * structure passed into its module start routine, as the
+ * <code>id</code> field (cast to this type).
+ */
+#endif
 typedef uint32_t  OSKextLoadTag;
 
 /*!
@@ -484,7 +611,6 @@ typedef uint32_t  OSKextLoadTag;
  */
 #define  kOSKextInvalidLoadTag  ((OSKextLoadTag)(-1))
 
-
 #ifdef KERNEL
 
 /* Make these visible to kexts only and *not* the kernel.
@@ -542,7 +668,7 @@ const char * OSKextGetCurrentVersionString(void);
  * @group Kext Loading C Functions
  * Functions for loading and tracking kexts in the kernel.
  */
- 
+
 /*!
  * @function OSKextLoadKextWithIdentifier
  *
@@ -671,6 +797,11 @@ OSReturn OSKextReleaseKextWithLoadTag(OSKextLoadTag loadTag);
 #pragma mark Kext Requests
 /********************************************************************/
 #endif
+/*!
+ * @group Kext Requests to User Space
+ * Functions for making requests to kextd in user space.
+ */
+
 /*!
  * @typedef OSKextRequestTag
  *
@@ -679,9 +810,15 @@ OSReturn OSKextReleaseKextWithLoadTag(OSKextLoadTag loadTag);
  */
 typedef uint32_t OSKextRequestTag;
 
+/*!
+ * @define kOSKextRequestTagInvalid
+ *
+ * @abstract
+ * A request tag value that will never be used for a kext request;
+ * indicates failure to create/queue the request.
+ */
 #define kOSKextRequestTagInvalid  ((OSKextRequestTag)-1)
 
-
 /*!
  * @typedef OSKextRequestResourceCallback
  *
@@ -710,11 +847,11 @@ typedef uint32_t OSKextRequestTag;
  *                             OSKextRequestResource@/link</code>.
  */
 typedef void (* OSKextRequestResourceCallback)(
-    OSKextRequestTag                requestTag,
-    OSReturn                        result,
-    const void                    * resourceData,
-    uint32_t                        resourceDataLength,
-    void                          * context);
+	OSKextRequestTag                requestTag,
+	OSReturn                        result,
+	const void                    * resourceData,
+	uint32_t                        resourceDataLength,
+	void                          * context);
 
 /*!
  * @function OSKextRequestResource
@@ -732,7 +869,10 @@ typedef void (* OSKextRequestResourceCallback)(
  *                         when it is invoked. May be <code>NULL</code>.
  * @param  requestTagOut   If non-<code>NULL</code>,
  *                         filled on success with a tag identifying the
- *                         pending request; can be used with
+ *                         pending request
+ *                         (or on failure with <code>@link kOSKextRequestTagInvalid
+ *                         kOSKextRequestTagInvalid@/link</code>;
+ *                         can be used with
  *                         <code>@link OSKextCancelRequest
  *                         OSKextCancelRequest@/link</code>.
  *
@@ -748,12 +888,23 @@ typedef void (* OSKextRequestResourceCallback)(
  * Other <code>OSKextReturn...</code> errors are possible.
  *
  * @discussion
- * This function queues a request to the user-space kext daemon
+ * This function queues an asynchronous request to the user-space kext daemon
  * <code>@link //apple_ref/doc/man/8/kextd kextd(8)@/link</code>;
  * requests for resources early in system startup
  * will not be fulfilled until that daemon starts.
- * Note also that the localization context of the kext daemon
- * (namely tha tof the superuser)
+ * Requests made by a kext while that kext is loading
+ * (specifically in the kext's module start routine)
+ * will not be fulfilled until after the start routine returns and
+ * the kext is completely loaded.
+ * Kexts requesting resources should be sure to perform appropriate locking
+ * in the callback function.
+ *
+ * Kext resources are stored in the kext's on-disk bundle under the
+ * Resources subdirectory.
+ * See {@linkdoc //apple_ref/doc/uid/10000123i Bundle Programming Guide}
+ * for an overview of bundle structure.
+ * The localization context of the kext daemon
+ * (namely that of the superuser)
  * will be used in retrieving resources;
  * kext resources intended for use in the kernel
  * should generally not be localized.
@@ -786,11 +937,11 @@ typedef void (* OSKextRequestResourceCallback)(
  * outside of the stop function.
  */
 OSReturn OSKextRequestResource(
-    const char                    * kextIdentifier,
-    const char                    * resourceName,
-    OSKextRequestResourceCallback   callback,
-    void                          * context,
-    OSKextRequestTag              * requestTagOut);
+	const char                    * kextIdentifier,
+	const char                    * resourceName,
+	OSKextRequestResourceCallback   callback,
+	void                          * context,
+	OSKextRequestTag              * requestTagOut);
 
 /*!
  * @function OSKextCancelRequest
@@ -824,11 +975,65 @@ OSReturn OSKextRequestResource(
  * before the stop function is called.
  */
 OSReturn OSKextCancelRequest(
-    OSKextRequestTag    requestTag,
-    void             ** contextOut);
+	OSKextRequestTag    requestTag,
+	void             ** contextOut);
 
 
-#if (__x86_64__)
+/*!
+ * @function OSKextGrabPgoData
+ *
+ * @abstract
+ * Grab a LLVM profile data buffer from a loaded kext.
+ *
+ * @param   uuid             the uuid identifying the kext to retrieve data from
+ * @param   pSize            pointer of where to store the size of the buffer.   May be NULL.
+ * @param   pBuffer          pointer to the output buffer.   May be NULL.
+ * @param   bufferSize       size of the buffer pointed to by pBuffer
+ * @param   wait_for_unload  (boolean) sleep until the kext is unloaded
+ * @param   metadata         (boolean) include metadata footer
+ *
+ * @result
+ * 0 on success
+ * ENOTSUP if the kext does not have profile data to retrieve.
+ * ENOTSUP if no kext with the given UUID is found
+ * ERRORS  if the provided buffer is too small
+ * EIO     internal error, such as if __llvm_profile_write_buffer_internal fails
+ */
+int
+OSKextGrabPgoData(uuid_t uuid,
+    uint64_t *pSize,
+    char *pBuffer,
+    uint64_t bufferSize,
+    int wait_for_unload,
+    int metadata);
+
+/*!
+ * @function OSKextResetPgoCountersLock
+ *
+ * @abstract
+ * Call this function before trapping into the debugger to call OSKextResetPgoCounters.
+ */
+void
+OSKextResetPgoCountersLock(void);
+
+/*!
+ * @function OSKextResetPgoCountersUnlock
+ *
+ * @abstract
+ * Call this function after trapping into the debugger to call OSKextResetPgoCounters.
+ */
+void
+OSKextResetPgoCountersUnlock(void);
+
+/*!
+ * @function OSKextResetPgoCounters
+ *
+ * @abstract Reset the PGO counters for all kexts.  Call only from debugger
+ * context, while holding OSKextResetPgoCountersLock().
+ */
+void
+OSKextResetPgoCounters(void);
+
 
 #if PRAGMA_MARK
 #pragma mark -
@@ -836,7 +1041,6 @@ OSReturn OSKextCancelRequest(
 #pragma mark Weak linking
 /********************************************************************/
 #endif
-
 /*!
  * @group Weak Linking
  * Support for weak references to symbols in kexts.
@@ -845,11 +1049,11 @@ OSReturn OSKextCancelRequest(
 /*!
  * @var gOSKextUnresolved
  *
- * @abstract 
+ * @abstract
  * The value to which a kext's unresolved, weakly-referenced symbols are bound.
  *
  * @discussion
- * A kext must test a weak symbol before using it.  A weak symbol 
+ * A kext must test a weak symbol before using it.  A weak symbol
  * is only safe to use if it is not equal to <code>gOSKextUnresolved</code>.
  *
  * Example for a weak symbol named <code>foo</code>:
@@ -863,23 +1067,23 @@ OSReturn OSKextCancelRequest(
  * @/textblock
  * </pre>
  */
-extern const void * gOSKextUnresolved;
+extern const void * const gOSKextUnresolved;
 
 /*!
  * @define OSKextSymbolIsResolved
  *
- * @abstract 
+ * @abstract
  * Checks whether a weakly-referenced symbol has been resolved.
  *
  * @param weak_sym   The weak symbol to be tested for resolution.
  *
- * @result 
- * <code>TRUE</code> if weak_sym is resolved, or <code>FALSE</code> 
+ * @result
+ * <code>TRUE</code> if weak_sym is resolved, or <code>FALSE</code>
  * if weak_sym is unresolved.
  *
  * @discussion
  * This is a convenience macro for testing if weak symbols are resolved.
- * 
+ *
  * Example for a weak symbol named <code>foo</code>:
  * <pre>
  * @textblock
@@ -894,7 +1098,24 @@ extern const void * gOSKextUnresolved;
 #define OSKextSymbolIsResolved(weak_sym)        \
     (&(weak_sym) != gOSKextUnresolved)
 
-#endif /* (__x86_64__) */
+
+#if CONFIG_KEC_FIPS
+
+#if PRAGMA_MARK
+#pragma mark -
+/********************************************************************/
+#pragma mark Kernel External Components for FIPS compliance
+/********************************************************************/
+#endif
+
+// Kernel External Components for FIPS compliance (KEC_FIPS)
+// WARNING - ath_hash is owned by the kernel, do not free
+typedef struct AppleTEXTHash {
+	int                 ath_version;// version of this structure (value is 1 or 2)
+	int             ath_length; // length of hash data
+	void *          ath_hash;   // hash extracted from AppleTextHashes dict
+} AppleTEXTHash_t;
+#endif // CONFIG_KEC_FIPS
 
 #endif /* KERNEL */
 
