@@ -2,7 +2,7 @@
  * Copyright (c) 1998-2000 Apple Inc. All rights reserved.
  *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_START@
- * 
+ *
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -11,10 +11,10 @@
  * unlawful or unlicensed copies of an Apple operating system, or to
  * circumvent, violate, or enable the circumvention or violation of, any
  * terms of an Apple operating system software license agreement.
- * 
+ *
  * Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this file.
- * 
+ *
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -22,23 +22,25 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- * 
+ *
  * @APPLE_OSREFERENCE_LICENSE_HEADER_END@
  */
 
 #ifndef _LIBKERN_OSKEXTLIBPRIVATE_H
 #define _LIBKERN_OSKEXTLIBPRIVATE_H
 
+
 #include <sys/cdefs.h>
 
 __BEGIN_DECLS
-#ifdef KERNEL
+
 #include <libkern/OSTypes.h>
 #include <mach/kmod.h>
+
+#ifdef KERNEL
 #include <mach/vm_types.h>
-#else
-#include <CoreFoundation/CoreFoundation.h>
 #endif /* KERNEL */
+
 __END_DECLS
 
 #include <libkern/OSReturn.h>
@@ -57,6 +59,18 @@ typedef uint8_t OSKextExcludeLevel;
 #define kOSKextExcludeKext  (1)
 #define kOSKextExcludeAll   (2)
 
+#define kIOCatalogManagementEntitlement "com.apple.private.security.iocatalog-management"
+#define kOSKextCollectionManagementEntitlement "com.apple.private.security.kext-collection-management"
+#define kOSKextOnlyBootKCManagementEntitlement "com.apple.private.security.only-bootkc-management"
+
+#define kOSKextCodelessKextLoadAddr (0x7FFFFFFFFFFFFFFFULL)
+
+#if XNU_TARGET_OS_OSX
+#define kIOKitDaemonName "kernelmanagerd"
+#else
+#define kIOKitDaemonName "driverkitd"
+#endif /* XNU_TARGET_OS_OSX */
+
 #if PRAGMA_MARK
 #pragma mark -
 /********************************************************************/
@@ -70,6 +84,57 @@ typedef uint8_t OSKextExcludeLevel;
  */
 #define kOSBundleHelperKey      "OSBundleHelper"
 
+/*!
+ * @define   kOSBundleDeveloperOnlyKey
+ * @abstract A boolean value indicating whether the kext should only load on
+ *           Developer devices.
+ */
+#define kOSBundleDeveloperOnlyKey               "OSBundleDeveloperOnly"
+
+/*!
+ * @define   kOSBundleRamDiskOnlyKey
+ * @abstract A boolean value indicating whether the kext should only load when
+ *           booted from a ram disk.
+ */
+#define kOSBundleRamDiskOnlyKey         "OSBundleRamDiskOnly"
+
+
+/*!
+ * @define   kAppleSecurityExtensionKey
+ * @abstract A boolean value indicating whether the kext registers
+ *           MACF hooks.
+ */
+#define kAppleSecurityExtensionKey      "AppleSecurityExtension"
+
+/*!
+ * @define   kAppleKernelExternalComponentKey
+ * @abstract A boolean value indicating whether the kext is vending kernel
+ *           KPI, and needs special loading behavior.
+ */
+#define kAppleKernelExternalComponentKey        "AppleKernelExternalComponent"
+
+/*!
+ * @define kOSKextInfoPlistDigestKey
+ * @abstract SHA-256 data of the kext's Info.plist
+ */
+#define kOSKextInfoPlistDigestKey       "_InfoPlistDigest"
+
+/*!
+ * @define kOSKextBundleCollectionTypeKey
+ * @abstract The type of collection in which a kext is linked. Possible
+ *           values: kKCTypePrimary, kKCTypeSystem, kKCTypeAuxiliary,
+ *                   kKCTypeCodeless
+ */
+#define kOSKextBundleCollectionTypeKey  "_BundleCollectionType"
+
+/*!
+ * @define kOSKextAuxKCAvailabilityKey
+ * @abstract boolean value: false if the kext is in the AuxKC and
+ *           is not loadable; true otherwise.
+ */
+#define kOSKextAuxKCAvailabilityKey     "_AuxKCAvailability"
+
+
 // properties found in the registry root
 #define kOSKernelCPUTypeKey             "OSKernelCPUType"
 #define kOSKernelCPUSubtypeKey          "OSKernelCPUSubtype"
@@ -85,31 +150,67 @@ typedef uint8_t OSKextExcludeLevel;
 #endif
 /*********************************************************************
 * In addition to the keys defined here, you will find:
-*   CFBundleIdentifier
-*   CFBundleVersion
-*   OSBundleCompatibleVersion
-*   OSKernelResource
-*   OSBundleInterface
+*   kCFBundleIdentifierKey
+*   kCFBundleVersionKey
+*   kOSBundleCompatibleVersionKey
+*   kOSBundleIsInterfaceKey
+*   kOSKernelResourceKey
 *********************************************************************/
+#define kOSBundleMachOHeadersKey                "OSBundleMachOHeaders"
+#define kOSBundleLogStringsKey                  "OSBundleLogStrings"
 #define kOSBundleCPUTypeKey                     "OSBundleCPUType"
 #define kOSBundleCPUSubtypeKey                  "OSBundleCPUSubtype"
 #define kOSBundlePathKey                        "OSBundlePath"
+#define kOSBundleExecutablePathKey              "OSBundleExecutablePath"
 #define kOSBundleUUIDKey                        "OSBundleUUID"
+#define kOSBundleTextUUIDKey                    "OSBundleTextUUID"
 #define kOSBundleStartedKey                     "OSBundleStarted"
 #define kOSBundlePrelinkedKey                   "OSBundlePrelinked"
 #define kOSBundleLoadTagKey                     "OSBundleLoadTag"
 #define kOSBundleLoadAddressKey                 "OSBundleLoadAddress"
 #define kOSBundleLoadSizeKey                    "OSBundleLoadSize"
+#define kOSBundleExecLoadAddressKey             "OSBundleExecLoadAddress"
+#define kOSBundleExecLoadSizeKey                "OSBundleExecLoadSize"
 #define kOSBundleWiredSizeKey                   "OSBundleWiredSize"
 #define kOSBundleDependenciesKey                "OSBundleDependencies"
 #define kOSBundleRetainCountKey                 "OSBundleRetainCount"
+#define kOSBundleCacheLoadAddressKey            "OSBundleCacheLoadAddress"
+// Kernel TEXT encompasses kexts
+#define kOSBundleKextsInKernelTextKey           "OSBundleKextsInKernelText"
+// OSKextCopyLoadedKextInfo includes non-started kexts when present:
+#define kOSBundleAllPrelinkedKey                "OSBundleAllPrelinked"
+// OSKextCopyDextsInfo states:
+#define kOSBundleDextStateKey                   "OSBundleDextState"
+#define kOSBundleDextStateActiveKey             "OSBundleDextStateActive"
+#define kOSBundleDextStateActiveLoadedKey       "OSBundleDextStateActiveLoaded"
+#define kOSBundleDextStateActiveUnloadedKey     "OSBundleDextStateActiveUnloaded"
+#define kOSBundleDextStatePendingUpgradeKey     "OSBundleDextStatePendingUpgrade"
 
+
+/* Dictionary of metaclass info keyed by classname.
+ */
 #define kOSBundleClassesKey                     "OSBundleClasses"
 
+#define kOSBundleDextUniqueIdentifierKey        "kOSBundleDextUniqueIdentifier"
+/* These are contained in kOSBundleClassesKey. kOSMetaClassSuperclassNameKey
+ * may be absent (for the root class).
+ */
 #define kOSMetaClassNameKey                     "OSMetaClassName"
 #define kOSMetaClassSuperclassNameKey           "OSMetaClassSuperclassName"
 #define kOSMetaClassTrackingCountKey            "OSMetaClassTrackingCount"
 
+#if PRAGMA_MARK
+#pragma mark -
+/********************************************************************/
+#pragma mark Kext Collection Type Keys
+/********************************************************************/
+#endif
+#define kKCTypePrimary   "Primary"
+#define kKCTypeSystem    "System"
+#define kKCTypeAuxiliary "Auxiliary"
+#define kKCTypeCodeless  "Codeless"
+#define kKCTypeAny       "Any"
+
 #if PRAGMA_MARK
 #pragma mark -
 /********************************************************************/
@@ -137,7 +238,7 @@ typedef uint8_t OSKextExcludeLevel;
  *   <li>A <b>level</b> from 0-7 in the lowest-order nibble (0x7).</li>
  *   <li>A flag bit in the lowest-order nibble (0x8) indicating whether
  *       log messages tied to individual kexts are always printed (1)
- *       or printed only if the kext has an 
+ *       or printed only if the kext has an
  *       @link //apple_ref/c/macro/kOSBundleEnableKextLoggingKey
  *       OSBundleEnableKextLogging@/link set to <code>true</code>.
  *   <li>A set of <b>activity flags</b> in the remaining nibbles (0xFFFFFFF0),
@@ -528,8 +629,8 @@ typedef uint32_t OSKextLogSpec;
  *           set to <code>true</code>.
  */
 #define kOSKextLogShowAllKextsFilter     ((OSKextLogSpec) \
-                                           (kOSKextLogShowAllFilter | \
-                                            kOSKextLogKextOrGlobalMask))
+	                                   (kOSKextLogShowAllFilter | \
+	                                    kOSKextLogKextOrGlobalMask))
 
 #if PRAGMA_MARK
 #pragma mark -
@@ -541,7 +642,7 @@ typedef uint32_t OSKextLogSpec;
  * @group Kext Version String Processing
  * Functions for working with kext versions and compatible versions.
  */
- 
+
 /*!
  * @typedef   OSKextVersion
  * @abstract  An encoded kext version that can be compared arithmetically.
@@ -635,11 +736,13 @@ OSKextVersion OSKextParseVersionString(const char * versionString);
  * the resulting string is "(invalid)".
  */
 Boolean OSKextVersionGetString(
-    OSKextVersion   aVersion,
-    char          * buffer,
-    uint32_t        bufferSize);
+	OSKextVersion   aVersion,
+	char          * buffer,
+	uint32_t        bufferSize);
 
 
+#define KOSBundleDextUniqueIdentifierMaxLength (1024)
+
 #ifdef KERNEL
 
 
@@ -650,18 +753,9 @@ Boolean OSKextVersionGetString(
 /********************************************************************/
 #endif
 #ifdef XNU_KERNEL_PRIVATE
-void kext_weak_symbol_referenced(void);
+void kext_weak_symbol_referenced(void) __abortlike;
 #endif /* XNU_KERNEL_PRIVATE */
 
-#if !(__x86_64__)
-
-extern const void *gOSKextUnresolved;
-
-#define OSKextSymbolIsResolved(weak_sym)        \
-    (&(weak_sym) != gOSKextUnresolved)
-
-#endif /* !(__x86_64__) */
-
 #if PRAGMA_MARK
 #pragma mark -
 /********************************************************************/
@@ -681,6 +775,17 @@ vm_map_t kext_get_vm_map(kmod_info_t * info);
 
 #ifdef XNU_KERNEL_PRIVATE
 
+#if CONFIG_DTRACE
+/*!
+ * @function OSKextRegisterKextsWithDTrace
+ * @abstract
+ * DTrace calls this once when it has started up so that the kext system
+ * will register any already-loaded kexts with it.
+ */
+void OSKextRegisterKextsWithDTrace(void);
+
+#endif /* CONFIG_DTRACE */
+
 /*!
  * @function kext_dump_panic_lists
  * @abstract Prints compacted lists of last unloaded & all loaded kexts
@@ -694,6 +799,12 @@ void kext_dump_panic_lists(int (*printf_func)(const char *fmt, ...));
 
 #ifdef XNU_KERNEL_PRIVATE
 
+/*!
+ * @define kOSKextReceiptQueried
+ * @abstract Whether or not the kext receipt has been successfully loaded.
+ */
+#define kOSKextReceiptQueried  "OSKextReceiptQueried"
+
 #if PRAGMA_MARK
 #pragma mark -
 /********************************************************************/
@@ -742,7 +853,7 @@ void kext_dump_panic_lists(int (*printf_func)(const char *fmt, ...));
  * when there are no C++ objects and the kext reference count is zero.
  */
 uint32_t OSKextGetLoadTagForBundleIdentifier(
-    const char * kextIdentifier);
+	const char * kextIdentifier);
 
 
 /*!
@@ -770,7 +881,7 @@ uint32_t OSKextGetLoadTagForBundleIdentifier(
  * OSKextRetainKextWithLoadTag@/link</code>
  * with its own load tag
  * (the <code>id</code> field of its <code>kmod_info_t</code> struct),
- * followed by 
+ * followed by
  * <code>@link //apple_ref/c/func/OSKextReleaseKextWithLoadTag
  * OSKextReleaseKextWithLoadTag@/link</code>;
  * this will schedule the kext for unload on a separate thread.
@@ -787,6 +898,141 @@ OSReturn OSKextUnloadKextWithLoadTag(uint32_t loadTag);
 
 #endif /* KERNEL */
 
+#if PRAGMA_MARK
+#pragma mark -
+/********************************************************************/
+#pragma mark Loaded Kext Summary
+/********************************************************************/
+#endif
+
+/*!
+ * @define kOSKextLoadedKextSummaryVersion
+ * @abstract The current version of the loaded kext summary headers.
+ */
+#define kOSKextLoadedKextSummaryVersion 2
+
+/*!
+ * @typedef OSKextLoadedKextSummary
+ * @abstract A structure that describes a loaded kext.
+ *
+ * @field name The kext's bundle identifier.
+ * @field uuid The kext's UUID;
+ * @field address The kext's load address.
+ * @field size The kext's load size.
+ * @field version The binary format (OSKextVersion) version of the kext.
+ * @field loadTag The kext's load tag.
+ * @field flags Internal tracking flags.
+ * @field reference_list who this refs (links on).
+ * @field text_exec_address The address of the __TEXT_EXEC segment (if it exists), otherwise __TEXT
+ * @field text_exec_size The size of the segment pointed to by text_address
+ *
+ * @discussion
+ * The OSKextLoadedKextSummary structure contains a basic set of information
+ * about the kext to facilitate kext debugging and panic debug log output.
+ */
+typedef struct _loaded_kext_summary {
+	char        name[KMOD_MAX_NAME];
+	uuid_t      uuid;
+	uint64_t    address;
+	uint64_t    size;
+	uint64_t    version;
+	uint32_t    loadTag;
+	uint32_t    flags;
+	uint64_t    reference_list;
+	uint64_t    text_exec_address;
+	size_t      text_exec_size;
+} OSKextLoadedKextSummary;
+
+/*!
+ * @typedef OSKextLoadedKextSummaryHeader
+ * @abstract A structure that describes the set of loaded kext summaries.
+ *
+ * @field version The version of the loaded kext summary structures.
+ * @field entry_size The size of each entry in summaries.
+ * @field numSummaries The number of OSKextLoadedKextSummary structures
+ *        following the header.
+ * @field summaries A convenience pointer to the array of summaries following
+ *        the header.
+ *
+ * @discussion
+ * The OSKextLoadedKextSummaryHeader describes the set of loaded kext summaries
+ * available for use by the debugger or panic log routine.
+ * The array of summaries contains one OSKextLoadedKextSummary for every kext
+ * that declares an executable and is not an interface to the kernel.
+ */
+typedef struct _loaded_kext_summary_header {
+	uint32_t version;
+	uint32_t entry_size;
+	uint32_t numSummaries;
+	uint32_t reserved; /* explicit alignment for gdb  */
+	OSKextLoadedKextSummary summaries[0];
+} OSKextLoadedKextSummaryHeader;
+
+/*!
+ * @var gLoadedKextSummaries
+ * @abstract The global pointer to the current set of loaded kext summaries.
+ */
+extern OSKextLoadedKextSummaryHeader * gLoadedKextSummaries;
+
+/*!
+ * @var gLoadedKextSummariesTimestamp
+ *
+ * @abstract This will be set to mach_absolute_time() around updates to
+ * gLoadedKextSummaries.  Ie. immediately before gLoadedKextSummaries is set to
+ * zero, and immediately after it is set to a new value.
+ */
+extern uint64_t gLoadedKextSummariesTimestamp;
+
+/*!
+ * @function OSKextLoadedKextSummariesUpdated
+ * @abstract Called when gLoadedKextSummaries has been updated.
+ *
+ * @discussion
+ * gLoadedKextSummaries is updated when a kext is loaded or unloaded.
+ * When the update is complete, OSKextLoadedKextSummariesUpdated is called.
+ * gdb can set a breakpoint on this function to detect kext loads and unloads.
+ */
+void OSKextLoadedKextSummariesUpdated(void);
+
+#ifdef XNU_KERNEL_PRIVATE
+
+extern const vm_allocation_site_t * OSKextGetAllocationSiteForCaller(uintptr_t address);
+extern uint32_t                     OSKextGetKmodIDForSite(const vm_allocation_site_t * site,
+    char * name, vm_size_t namelen);
+extern void                         OSKextFreeSite(vm_allocation_site_t * site);
+
+extern kern_return_t                OSKextSetReceiptQueried(void);
+
+#if DEVELOPMENT || DEBUG
+extern void                         OSKextGetRefGrpForCaller(uintptr_t address,
+    void (^)(struct os_refgrp *));
+#endif
+
+#if CONFIG_IMAGEBOOT
+extern int OSKextGetUUIDForName(const char *, uuid_t);
+#endif
+
+extern vm_tag_t gIOSurfaceTag;
+
+extern void *OSKextKextForAddress(const void *addr);
+
+/*!
+ * @function OSKextGetLoadedKextSummaryForAddress
+ * @abstract Given an address, retrieve the summary of the kext which contains it.
+ *
+ * @discussion
+ * This function invokes OSKext::summaryForAddressExt, which will copy into the
+ * caller-provided pointer the summary of the kext containing the given address.
+ * This is done while holding the sKextSummariesLock lock, thus making it possible
+ * to use the content of the summary even if gLoadedKextSummaries is reallocated
+ * in the meantime.
+ */
+extern kern_return_t OSKextGetLoadedKextSummaryForAddress(
+	const void              * addr,
+	OSKextLoadedKextSummary * summary);
+
+#endif /* XNU_KERNEL_PRIVATE */
+
 __END_DECLS
 
 #endif /* ! _LIBKERN_OSKEXTLIBPRIVATE_H */
